require 'sketchup' # First we pull in the standard API hooks.
require 'Practica//lib//utils.rb'

#Class that establishes a lexicographic ordering of the points
class OrderedPoint
	
	#3D point
	attr_accessor :point

	#point:: a 3D point
	#
	#Initializes the ordered point
	def initialize(point)
		@point = point
	end
	
	#other_point:: another OrderedPoint
	#returns:: true if this point is lesser than other_point
	def < (other_point)
		result = (@point.x < other_point.x) || ((@point.x == other_point.x)&&(@point.y < other_point.y)) || ((@point.x == other_point.x)&&(@point.y == other_point.y)&&(@point.z < other_point.z))
		return result
	end
	
	#other_point:: another OrderedPoint
	#returns:: true if this point is equal to other_point
	def == (other_point)
		result = false
		if other_point.kind_of? OrderedPoint 
			if ((@point.x == other_point.x) && (@point.y == other_point.y)&&(@point.z == other_point.z))
				result = true
			end
		end
		return result	
	end
	
	#other_point:: another OrderedPoint
	#returns:: true if this point is bigger than other_point
	def > (other_point)
		return (other_point < self)
	end
	
	#other_point:: another OrderedPoint
	#returns:: true if this point is lesser or equal than other_point
	def <= (other_point)
		less = (self < other_point)
		equal = (self == other_point)
		return (less || equal)
	end
	
	#other_point:: another OrderedPoint
	#returns:: true if this point is bigger or equal than other_point
	def >= (other_point)
		great = (other_point < self)
		equal = (self== other_point)
		return (great || equal)
	end
	
	#x:: x coordinate
	#
	#returns:: true if the x-coordinate of this point is equal to x
	def x=(x)
		@point.x = x
	end
	
	#y:: y coordinate
	#
	#returns:: true if the y-coordinate of this point is equal to y
	def y=(x)
		@point.y = x
	end
	
	#z:: z coordinate
	#
	#returns:: true if the z-coordinate of this point is equal to z
	def z=(x)
		@point.z = x
	end
	
	#returns:: the x coordinate
	def x()
		return @point.x
	end
	
	#returns:: the y coordinate
	def y()
		return @point.y
	end
	
	#retuns:: the z coordinate
	def z()
		return @point.z
	end
	
	#returns:: an OrderedPoint object identical to this
	def clone()
		return OrderedPoint.new(@point.clone)
	end
	
	#other_point: another OrderedPoint
	#returns: the sum of this point and other_point
	def +(other_point)
		x = @point.x + other_point.x
		y = @point.y + other_point.y
		z = @point.z + other_point.z
		point = OrderedPoint.new(Geom::Point3d.new(x,y,z))
		return point
	end
	
	#other_point:: another OrderedPoint
	#
	#returns:: true if this point is equal to other_point
	def eql?(other_point)
		result = false
		if other_point.kind_of? OrderedPoint
			if ((@point.x == other_point.x) && (@point.y == other_point.y)&&(@point.z == other_point.z))
				result = true
			end
		end
		return result		
	end
	
	#other_point:: another OrederedPoint
	#
	#returns:: 1 if this point is bigger than other_point, -1 if this point is lesser than other_point or 0 if this point is equal to other_point
	def <=> (other_point)
		if self > other_point
			return 1
		elsif self < other_point
			return -1
		else
			return 0
		end
	end
end

#This class creates a cube
class Cube
	#The type of the shape: always cube
	attr_accessor :type
	
	#Cube side
	attr_accessor :size
	
	#The SU objects that groups the units
	attr_accessor :group
	
	#The base of the cube
	attr_accessor :base
	
	
	#Initializes the cube
	def initialize(cube_size)
		@size = cube_size
		@type = "cube"
		@group = nil
	end
	
	#transformation:: It's a Transformation objet that represents the position and rotation where we want to paint the cube
	#
	#Paints the cube
	def paint (transformation)
		@group = Sketchup.active_model.entities.add_group
		pts = [[0,0,0],[@size,0,0],[@size,@size,0],[0,@size,0],[0,0,0]]
		@base = @group.entities.add_face pts
		@base.pushpull Constants::HEIGHT_S
		@group.transformation = transformation
	end
	
	#returns:: a Cube object identical to this
	def duplicate()
		cube = Cube.new(@size)
		return cube
	end
	
	#Deletes the cube
	def erase
		Sketchup.active_model.entities.erase_entities @group
		Practica.project.modifying = false
		Practica.project.erasing = false
		@group = nil
	end
end

#This class creates a prism
class TriangularPrism
	#The type of the shape: always triangle
	attr_accessor :type
	
	#Triangle side
	attr_accessor :size
	
	#The SU objects that groups the units
	attr_accessor :group
	
	#Initializes the prism
	def initialize(prism_size)
		@size = prism_size
		@group = nil
		@type = "triangle"
	end
	
	#transformation:: It's a Transformation objet that represents the position and rotation where we want to paint the triangular prism
	#
	#Paints the triangular prism
	def paint (transformation)
		@group = Sketchup.active_model.entities.add_group
		pts = [[0,0,0],[@size,0,0],[@size/2,Constants::HEIGHT_T,0],[0,0,0]]
		@base = @group.entities.add_face pts
		@base.pushpull Constants::HEIGHT_S
		@group.transformation = transformation
	end
	
	#returns:: a triangular prism object identical to this
	def duplicate()
		prism = TriangularPrism.new(@size)
		return prism
	end
	
	#Deletes the triangular prism
	def erase
		Sketchup.active_model.entities.erase_entities @group
		Practica.project.modifying = false
		Practica.project.erasing = false
		@group = nil
	end
end

#This class creates a hexagonal prism
class HexagonalPrism
	#The type of the shape: always hexagonal
	attr_accessor :type
	
	#Hexagonal side
	attr_accessor :size
	
	#The SU objects that groups the units
	attr_accessor :group
	
	#Initializes the prism
	def initialize(prism_size)
		@size = prism_size
		@group = nil
		@type = "hexagonal"
	end
	
	#transformation:: It's a Transformation objet that represents the position and rotation where we want to paint the hexagonal prism
	#
	#Paints the hexagonal prism
	def paint (transformation)
		@group = Sketchup.active_model.entities.add_group
		pts = [[0,0,0],[@size,0,0],[@size+(@size/2),Constants::APOTEMA,0],[@size,2*Constants::APOTEMA,0],[0,2*Constants::APOTEMA,0],[-@size/2,Constants::APOTEMA,0],[0,0,0]]
		@base = @group.entities.add_face pts
		@base.pushpull Constants::HEIGHT_S
		@group.transformation = transformation
	end
	
	#returns:: an hexagonal prism object identical to this
	def duplicate()
		prism = HexagonalPrism.new(@size)
		return prism
	end
	
	#Deletes the cube
	def erase
		Sketchup.active_model.entities.erase_entities @group
		Practica.project.modifying = false
		Practica.project.erasing = false
		@group = nil
	end
end

#This class represents a cube with a position
class PositionCube
	#A cube  object
	attr_accessor :cube
	
	#The position of the cube
	attr_accessor :position
	
	#The type of the shape: always cube
	attr_accessor :type
	
	#cube:: A cube objet
	#position:: an OrderedPoint
	def initialize(cube, position)
		@cube = cube
		#Class OrderedPoint
		@position = position
		@type = "cube"
	end
	
	#returns:: true if the cube is inside an area
	def area
		solution = false
		cond1 = (Constants::VECTOR_TRANSLATION_AXIOM[0][0] + @position.x).to_m.m
		cond2 = (Constants::VECTOR_TRANSLATION_AXIOM[0][1] + @position.y).to_m.m
		cond3 = (Constants::VECTOR_TRANSLATION_AXIOM[0][0] + @position.x + @cube.size).to_m.m
		cond4 = (@cube.size + Constants::VECTOR_TRANSLATION_AXIOM[0][1] + @position.y).to_m.m
		if ((cond1 >= Constants::PTS_H1[0][0])&&(cond2 >= Constants::PTS_H2[1][1]))
			if ( cond3 <= Constants::PTS_H2[1][0]) && @position.z >=0
				if (cond4 <= Constants::PTS_H1[0][1])
					solution = true
				end
			end
		end
		return solution
	end
	
	#returns:: true if the cube is inside a pyramide whose base is 100 metres by 100 metres and whose angle is 30 degrees
	def dimension
		solution = false
			#Now we teste if the shape isn't higher than the pyramid
			x_position = (Constants::VECTOR_TRANSLATION_AXIOM[0][0] + @position.x - Constants::PTS_H1[0][0]).to_m.m
			y_position = -(Constants::VECTOR_TRANSLATION_AXIOM[0][1] + @position.y - Constants::PTS_H1[0][1]).to_m.m
			if(x_position >Constants::SIZE_P/2)
				x_position = (Constants::SIZE_P - x_position).to_m.m
				x_position = (x_position - Constants::SIZE).to_m.m
				height = (x_position*Constants::ALFA).to_m.m
				final = (@position.z + Constants::HEIGHT_S).to_m.m
				if ((final> height)||(@position.z<0))
					solution = false
				else
					if(y_position >Constants::SIZE_P/2)
						y_position = (Constants::SIZE_P - y_position).to_m.m
						height = (y_position*Constants::ALFA).to_m.m
						final = (@position.z + Constants::HEIGHT_S).to_m.m
						if ((final> height)||(@position.z<0))
							solution = false
						else
							solution = true
						end
					else
						y_position = (y_position - Constants::SIZE).to_m.m
						final = (@position.z + Constants::HEIGHT_S).to_m.m
						height = y_position*Constants::ALFA
						if((final>height)||(@position.z<0))
							solution = false
						else
							solution = true
						end
					end
				end
			else
				final = (@position.z + Constants::HEIGHT_S).to_m.m
				height = x_position*Constants::ALFA
				if((final>height)||(@position.z<0))
					solution = false
				else
					if(y_position >Constants::SIZE_P/2)
						y_position = (Constants::SIZE_P - y_position).to_m.m
						height = (y_position*Constants::ALFA).to_m.m
						final = (@position.z + Constants::HEIGHT_S).to_m.m
						if ((final> height)||(@position.z<0))
							solution = false
						else
							solution = true
						end
					else
						y_position = (y_position - Constants::SIZE).to_m.m
						final = (@position.z + Constants::HEIGHT_S).to_m.m
						height = y_position*Constants::ALFA
						if((final>height)||(@position.z<0))
							solution = false
						else
							solution = true
						end
					end
				end
			end
		return solution
	end
	
	#Deletes the cube
	def erase
		@cube.erase
	end
	
	#candidate:: other PositionCube
	#
	#returns:: true if candidate is in the same position as @cube
	def == (candidate)
		equal = false
		if @position == candidate.position
			equal = true
		end
		return equal
	end
end

#This class represents a triangular prism with a position
class PositionTriangularPrism
	#A TriangularPrism object
	attr_accessor :prism
	
	#The position of the @prism
	attr_accessor :position
	
	#Indicates if we've a turned trianble
	attr_accessor :turned
	
	#The type of the shape: always triangle
	attr_accessor :type
	
	#prism:: A TriangularPrism objet
	#position:: an OrderedPoint
	def initialize(prism, position, turned)
		@prism = prism
		#Class OrderedPoint
		@position = position
		@turned = turned
		@type = "triangle"
	end
	
	#returns:: true if the triangular prism is inside an area
	def area
		solution = false
		if turned
			z_position = (Constants::VECTOR_TRANSLATION_AXIOM[0][2] + @position.z)
		else
			z_position = @position.z
		end
		cond1 = (Constants::VECTOR_TRANSLATION_AXIOM[0][0] + @position.x).to_m.m
		if !@turned
			cond2 = (Constants::VECTOR_TRANSLATION_AXIOM[0][1] + @position.y).to_m.m
		else
			cond2 = (Constants::VECTOR_TRANSLATION_AXIOM[0][1] + @position.y - Constants::HEIGHT_T).to_m.m
		end
		cond3 = (Constants::VECTOR_TRANSLATION_AXIOM[0][0] + @position.x + @prism.size).to_m.m
		if !@turned
			cond4 = (Constants::HEIGHT_T + Constants::VECTOR_TRANSLATION_AXIOM[0][1] + @position.y).to_m.m
		else
			cond4 = (Constants::VECTOR_TRANSLATION_AXIOM[0][1] + @position.y).to_m.m
		end
		if ((cond1 >= Constants::PTS_H1[0][0])&&(cond2 >= Constants::PTS_H2[1][1])) 
			if ( cond3 <= Constants::PTS_H2[1][0])
				if (cond4 <= Constants::PTS_H1[0][1])&& z_position >=0
					solution = true
				end
			end
		end
		return solution
	end
	
	#returns:: true if the triangular prism is inside a pyramide whose base is 100 metres by 100 metres and whose angle is 30 degrees
	def dimension
		solution = false
			#Now we teste if the shape isn't higher than the pyramid
			x_position = (Constants::VECTOR_TRANSLATION_AXIOM[0][0] + @position.x - Constants::PTS_H1[0][0]).to_m.m
			y_position = -(Constants::VECTOR_TRANSLATION_AXIOM[0][1] + @position.y - Constants::PTS_H1[0][1]).to_m.m
			if turned
				z_position = (Constants::VECTOR_TRANSLATION_AXIOM[0][2] + @position.z)
			else
				z_position = @position.z
			end
			if(x_position >Constants::SIZE_P/2)
				x_position = (Constants::SIZE_P - x_position).to_m.m
				x_position = (x_position - Constants::SIZE).to_m.m
				height = (x_position*Constants::ALFA).to_m.m
				final = (z_position + Constants::HEIGHT_S).to_m.m
				if ((final> height)||(z_position<0))
					solution = false
				else
					if(y_position >Constants::SIZE_P/2)
						y_position = (Constants::SIZE_P - y_position).to_m.m
						if @turned
							y_position = (y_position - Constants::HEIGHT_T).to_m.m
						end
						height = (y_position*Constants::ALFA).to_m.m
						final = (z_position + Constants::HEIGHT_S).to_m.m
						if ((final> height)||(z_position<0))
							solution = false
						else
							solution = true
						end
					else
						if !@turned
							y_position = (y_position - Constants::HEIGHT_T).to_m.m
						end
						final = (z_position + Constants::HEIGHT_S).to_m.m
						height = y_position*Constants::ALFA
						if((final>height)||(z_position<0))
							solution = false
						else
							solution = true
						end
					end
				end
			else
				final = (z_position + Constants::HEIGHT_S).to_m.m
				height = x_position*Constants::ALFA
				if((final>height)||(z_position<0))
					solution = false
				else
					if(y_position >Constants::SIZE_P/2)
						y_position = (Constants::SIZE_P - y_position).to_m.m
						if @turned
							y_position = (y_position - Constants::HEIGHT_S).to_m.m
						end
						height = (y_position*Constants::ALFA).to_m.m
						final = (z_position + Constants::HEIGHT_S).to_m.m
						if ((final> height)||(z_position<0))
							solution = false
						else
							solution = true
						end
					else
						if !@turned
							y_position = (y_position - Constants::HEIGHT_T).to_m.m
						end
						final = (z_position + Constants::HEIGHT_S).to_m.m
						height = y_position*Constants::ALFA
						if((final>height)||(z_position<0))
							solution = false
						else
							solution = true
						end
					end
				end
			end
		return solution
	end
	
	#Deletes the triangular prism
	def erase
		@prism.erase
	end
	
	#candidate:: other PositionTriangularPrism
	#
	#returns:: true if candidate is in the same position as @prism
	def == (candidate)
		equal = false
		if (@position == candidate.position)&&(@turned == candidate.turned)
			equal = true
		end
		return equal
	end
end

#This class represents a hexagonal prism with a position
class PositionHexagonalPrism
	#A HexagonalPrism object
	attr_accessor :prism
	
	#The position of the @prism
	attr_accessor :position
	
	#The type of the shape: always hexagonal
	attr_accessor :type
	
	#prism:: An HexagonalPrism object
	#position:: an OrderedPoint
	def initialize(prism, position)
		@prism = prism
		#Class OrderedPoint
		@position = position
		@type = "hexagonal"
	end
	
	#returns:: true if the hexagonal prism is inside an area
	def area
		solution = false
		cond1 = (Constants::VECTOR_TRANSLATION_AXIOM[0][0] + @position.x - @prism.size/2).to_m.m
		cond2 = (Constants::VECTOR_TRANSLATION_AXIOM[0][1] + @position.y).to_m.m
		cond3 = (Constants::VECTOR_TRANSLATION_AXIOM[0][0] + @position.x + @prism.size + @prism.size/2).to_m.m
		cond4 = (2*Constants::APOTEMA + Constants::VECTOR_TRANSLATION_AXIOM[0][1] + @position.y).to_m.m
		if ((cond1 >= Constants::PTS_H1[0][0])&&(cond2 >= Constants::PTS_H2[1][1]))
			if (cond3 <= Constants::PTS_H2[1][0])
				if (cond4 <= Constants::PTS_H1[0][1]) && @position.z >= 0
					solution = true
				end
			end
		end
		return solution
	end
	
	#returns:: true if the hexagonal prism is inside a pyramide whose base is 100 metres by 100 metres and whose angle is 30 degrees
	def dimension
		solution = false
		#Now we teste if the shape isn't higher than the pyramid
		x_position = (Constants::VECTOR_TRANSLATION_AXIOM[0][0] + @position.x - Constants::PTS_H1[0][0]).to_m.m
		y_position = -(Constants::VECTOR_TRANSLATION_AXIOM[0][1] + @position.y - Constants::PTS_H1[0][1]).to_m.m
		if(x_position >Constants::SIZE_P/2)
			x_position = (Constants::SIZE_P - x_position).to_m.m
			x_position = (x_position - @prism.size).to_m.m
			height = (x_position*Constants::ALFA).to_m.m
			final = (@position.z + Constants::HEIGHT_S).to_m.m
			if ((final> height)||(@position.z<0))
				solution = false
			else
				if(y_position >Constants::SIZE_P/2)
					y_position = (Constants::SIZE_P - y_position).to_m.m
					height = (y_position*Constants::ALFA).to_m.m
					final = (@position.z + Constants::HEIGHT_S).to_m.m
					if ((final> height)||(@position.z<0))
						solution = false
					else
						solution = true
					end
				else
					y_position = (y_position - 2*Constants::APOTEMA).to_m.m
					final = (@position.z + Constants::HEIGHT_S).to_m.m
					height = y_position*Constants::ALFA
					if((final>height)||(@position.z<0))
						solution = false
					else
						solution = true
					end
				end
			end
		else
			final = (@position.z + Constants::HEIGHT_S).to_m.m
			height = x_position*Constants::ALFA
			if((final>height)||(@position.z<0))
				solution = false
			else
				if(y_position >Constants::SIZE_P/2)
					y_position = (Constants::SIZE_P - y_position).to_m.m
					height = (y_position*Constants::ALFA).to_m.m
					final = (@position.z + Constants::HEIGHT_S).to_m.m
					if ((final> height)||(@position.z<0))
						solution = false
					else
						solution = true
					end
				else
					y_position = (y_position - 2*Constants::APOTEMA).to_m.m
					final = (@position.z + Constants::HEIGHT_S).to_m.m
					height = y_position*Constants::ALFA
					if((final>height)||(@position.z<0))
						solution = false
					else
						solution = true
					end
				end
			end
		end
		return solution
	end
	
	#Deletes the hexagonal prism
	def erase
		@prism.erase
	end
	
	#candidate:: other PositionHesagonalPrism
	#
	#returns:: true if candidate is in the same position as @prism
	def == (candidate)
		equal = false
		if @position == candidate.position
			equal = true
		end
		return equal
	end
end
		


#This class represents the design when it's composed by cubes
class CubeShape
	#Array of PositionCube
	attr_accessor :c
	
	#The type of the shape: always cube
	attr_accessor :type
	
	attr_accessor :occupation1
	
	#Array with all the free point positions
	attr_reader :point
	#Array with all the free face positions
	attr_reader :face
	#Array with all the free edge positions
	attr_reader :edge
	#Array with all the free  faceup positions
	attr_reader :faceup
	#Array with all the free edgeup positions
	attr_reader :edgeup
	#Array with all the free faceleft positions
	attr_reader :faceleft
	#Array with all the free edgeleft positions
	attr_reader :edgeleft
	
	#cube:: An Cube object
	#position:: an OrderedPoint
	def initialize(cube, position)
		@c = Array.new
		@c.push(PositionCube.new(cube, position))
		@face = [OrderedPoint.new(Geom::Point3d.new(cube.size,0,0)), OrderedPoint.new(Geom::Point3d.new(-cube.size,0,0)),OrderedPoint.new(Geom::Point3d.new(0,cube.size,0)), OrderedPoint.new(Geom::Point3d.new(0,-cube.size,0)), OrderedPoint.new(Geom::Point3d.new(0,0,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,0,-Constants::HEIGHT_S))]
		@face.sort
		@faceup = [OrderedPoint.new(Geom::Point3d.new(0,0,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,0,-Constants::HEIGHT_S))]
		@faceup.sort
		@faceleft = [OrderedPoint.new(Geom::Point3d.new(0,cube.size,0)), OrderedPoint.new(Geom::Point3d.new(0,-cube.size,0)), OrderedPoint.new(Geom::Point3d.new(cube.size,0,0)), OrderedPoint.new(Geom::Point3d.new(-cube.size,0,0))]
		@faceleft.sort
		@edge =  [OrderedPoint.new(Geom::Point3d.new(cube.size,-cube.size,0)), OrderedPoint.new(Geom::Point3d.new(-cube.size,-cube.size,0)), OrderedPoint.new(Geom::Point3d.new(cube.size,cube.size,0)), OrderedPoint.new(Geom::Point3d.new(-cube.size,cube.size,0)), OrderedPoint.new(Geom::Point3d.new(0,-cube.size,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,cube.size,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,-cube.size,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,cube.size,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(cube.size,0,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(cube.size,0,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-cube.size,0,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-cube.size,0,-Constants::HEIGHT_S))]
		@edge.sort
		@edgeup =  [OrderedPoint.new(Geom::Point3d.new(0,-cube.size,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,cube.size,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,-cube.size,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,cube.size,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(cube.size,0,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(cube.size,0,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-cube.size,0,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-cube.size,0,-Constants::HEIGHT_S))]
		@edgeup.sort
		@edgeleft =  [OrderedPoint.new(Geom::Point3d.new(cube.size,-cube.size,0)), OrderedPoint.new(Geom::Point3d.new(-cube.size,-cube.size,0)), OrderedPoint.new(Geom::Point3d.new(cube.size,cube.size,0)), OrderedPoint.new(Geom::Point3d.new(-cube.size,cube.size,0))]
		@edgeleft.sort
		@point = [OrderedPoint.new(Geom::Point3d.new(cube.size,cube.size,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(cube.size,cube.size,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(cube.size,-cube.size,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(cube.size,-cube.size,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-cube.size,-cube.size,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-cube.size,-cube.size,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-cube.size,cube.size,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-cube.size,cube.size,-Constants::HEIGHT_S))]
		@point.sort
		@type = "cube"
	end
	
	#Repaints the view of the shape
	def repaint
		@c.each { |cube_shape|
			if !cube_shape.cube.group
				transfor = Constants::AXIOM_T
				t_position = Geom::Transformation.new(Geom::Point3d.new(cube_shape.position.x, cube_shape.position.y, cube_shape.position.z))
				transfor = transfor * t_position
				cube_shape.cube.paint(transfor)
			end
		}
	end
	
	#Deletes the current shape
	def erase
		@c.each { |cube_shape|
			cube_shape.erase
		}
		@c.clear
	end
	
	#returns:: a CubeShape object identical to this
	def duplicate
		shape = CubeShape.new(@c[0].cube.duplicate, @c[0].position)
		cont = 1
		while cont<@c.length
			shape.c.push(PositionCube.new(@c[cont].cube.duplicate, @c[cont].position))
			cont+=1
		end
		return shape
	end
	
	#position:: an OrderedPoint
	#
	#returns:: the array position where there is a shape with the same position
	def position (position)
		cont = 0
		found = false
		while ((cont < @c.length)&&(!found))
			if @c[cont].position == position
				found = true
			else
				cont+=1
			end
		end
		return cont
	end
	
	#candidate:: A CubePosition object
	#
	#returns:: true if candidate is in CubeShape
	def esta (candidate)
		cont = 0
		find = false
		while ((cont < @c.length)&&(!find))
			if @c[cont].position == candidate.position
				find = true
			end
			cont+=1
		end
		return find
	end
	
	#position:: an OrderedPoint
	#
	#returns:: true if we have a cube in position
	def esta_pos (position)
		cont = 0
		find = false
		while ((cont < @c.length)&&(!find))
			if @c[cont].position == position
				find = true
			end
			cont+=1
		end
		return find
	end
	
	def actual_occupation
		if @occupation1
			@occupation1.clear
		else
			@occupation1 = Array.new
		end
		@c.each {|shape|
			position = shape.position.clone
			position.z = 0
			if !@occupation1.include? position
				@occupation1.push position
			end
		}
		
	end
	
	def occupation (shape1)
		solution = false
		position = shape1.position.clone
		position.z = 0
		cont = @occupation1.length
		if !@occupation1
			@occupation1 = Array.new
			cont += 1
		elsif !@occupation1.include? position
			cont += 1
		end
		area = Constants::SIZE*Constants::SIZE*cont
		area_solar_percent = (Constants::SIZE_P*Constants::SIZE_P*Constants::OCCUPATION)/100
		if area <= area_solar_percent
			solution = true
		end
		return solution
	end
	
	#path:: path to save the shape in
	#
	#Saves the shape in the specified path
	def save(path)
		File.open(path, 'w') do |f|
			f.write("Type: #{@type}\n")
			@c.each{|shape|
				f.write("S: #{shape.position.x.to_f.to_m} #{shape.position.y.to_f.to_m} #{shape.position.z.to_f.to_m}\n")
			}
		end
	end
	
	#path:: path to load the shape from
	#
	#Loads the shape in the specified path
	def load(path)
		current = self.duplicate
		self.erase
		salir = false
		File.open(path, 'r') do |f|
			while (line = f.gets)&&(!salir)
				line_a1 = line.split(":")
				if line_a1[0].include? "Type"
					@type = line_a1[1]
				elsif line_a1[0] == 'S'
					line_a = line_a1[1].split
					point = OrderedPoint.new(Geom::Point3d.new(line_a[0].to_f.m, line_a[1].to_f.m, line_a[2].to_f.m))
					cube = Cube.new(Constants::SIZE)
					shape = PositionCube.new(cube, point)
					@c.push(shape)
				else
					salir = true
					@c = current.c
					UI.messagebox("El archivo no tiene la estructura interna deseada")
				end
			end
		end
		self.repaint
	end
	
	#current:: a CubeShape objet
	#
	#returns:: true if this CubeShape is equal to current
	def repe(current)
		igual = true
		cont = 0
		if current.c.length == @c.length
			while (cont < current.c.length)&&igual
				if !esta(current.c[cont])
					igual = false
				end
				cont+=1
			end
		else
			igual = false
		end
		return igual
	end
end

#This class represents the design when it's composed by triangular prisms
class TriangularPrismShape
	#Array, for each prism I have 
	attr_accessor :c
	
	#The type of the shape: always triangular
	attr_accessor :type
	
	attr_accessor :occupation1
	
	#Array with all the free point positions
	attr_reader :point
	#Array with all the free turnedpoint positions
	attr_reader :turnedpoint
	#Array with all the free face positions
	attr_reader :face
	#Array with all the free turnedface positions
	attr_reader :turnedface
	#Array with all the free edge positions
	attr_reader :edge
	#Array with all the free turnededge positions
	attr_reader :turnededge
	#Array with all the free turnededge2 positions
	attr_reader :turnededge2
	
	#prism:: An TriangularPrism object
	#position:: an OrderedPoint
	#turned:: true if we've a turned triangle
	def initialize(prism, position, turned)
		@c = Array.new
		@c.push(PositionTriangularPrism.new(prism, position, turned))
		@turnedface = [OrderedPoint.new(Geom::Point3d.new(prism.size/2,Constants::HEIGHT_T,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size/2,Constants::HEIGHT_T,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,0,-prism.size))]
		@turnedface.sort
		@face = [OrderedPoint.new(Geom::Point3d.new(0,0,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,0,-Constants::HEIGHT_S))]
		@face.sort
		@edge =  [OrderedPoint.new(Geom::Point3d.new(prism.size,0,0)), OrderedPoint.new(Geom::Point3d.new(-prism.size,0,0)), OrderedPoint.new(Geom::Point3d.new(prism.size/2,Constants::HEIGHT_T,0)), OrderedPoint.new(Geom::Point3d.new(-prism.size/2,Constants::HEIGHT_T,0)), OrderedPoint.new(Geom::Point3d.new(prism.size/2,-Constants::HEIGHT_T,0)), OrderedPoint.new(Geom::Point3d.new(-prism.size/2,-Constants::HEIGHT_T,0))]
		@edge.sort
		@turnededge = [OrderedPoint.new(Geom::Point3d.new(0,0,0)), OrderedPoint.new(Geom::Point3d.new(0,0,-2*Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(prism.size/2,Constants::HEIGHT_T,-2*Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size/2,Constants::HEIGHT_T,-2*Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(prism.size/2,Constants::HEIGHT_T,0)), OrderedPoint.new(Geom::Point3d.new(-prism.size/2,Constants::HEIGHT_T,0))]
		@turnededge.sort
		@point = [OrderedPoint.new(Geom::Point3d.new(prism.size,0,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(prism.size,0,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size,0,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size,0,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(prism.size/2,Constants::HEIGHT_T,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(prism.size/2,Constants::HEIGHT_T,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size/2,Constants::HEIGHT_T,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size/2,Constants::HEIGHT_T,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-Constants::SIZE/2,-Constants::HEIGHT_T,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-Constants::SIZE/2,-Constants::HEIGHT_T,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(Constants::SIZE/2,-Constants::HEIGHT_T,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(Constants::SIZE/2,-Constants::HEIGHT_T,-Constants::HEIGHT_S))]
		@point.sort
		@turnedpoint = [OrderedPoint.new(Geom::Point3d.new(prism.size,0,-2*Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size,0,-2*Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(prism.size,0,0)), OrderedPoint.new(Geom::Point3d.new(-prism.size,0,0))]
		@turnedpoint.sort
		@turnededge2 = [OrderedPoint.new(Geom::Point3d.new(prism.size,0,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size,0,-Constants::HEIGHT_S))]
		@turnededge2.sort
		@type = "triangle"
	end
	
	#Repaints the view of the shape
	def repaint
		@c.each { |prism_shape|
			if !prism_shape.prism.group
				transfor = Constants::AXIOM_T
				t_position = Geom::Transformation.new
				t_position = [prism_shape.position.x, prism_shape.position.y, prism_shape.position.z]
				transfor = transfor * t_position
				prism_shape.prism.paint(transfor)
				if (prism_shape.turned)
					vector = Geom::Vector3d.new(1,0,0)
					transformation = Geom::Transformation.rotation transfor, vector, Math::PI
					prism_shape.prism.group.transform! transformation
				end
			end
		}
	end
	
	#Deletes the current shape
	def erase
		@c.each { |prism_shape|
			prism_shape.erase
		}
		@c.clear
	end
	
	#returns:: a TriangularPrismShape object identical to this
	def duplicate
		shape = TriangularPrismShape.new(@c[0].prism, @c[0].position, c[0].turned)
		cont = 1
		while cont<@c.length
			shape.c.push(PositionTriangularPrism.new(@c[cont].prism.duplicate, @c[cont].position, @c[cont].turned))
			cont+=1
		end
		shape.type = @type
		return shape
	end
	
	#position:: an OrderedPoint
	#turned:: true/false
	#
	#returns:: the array position where there is a shape with the same position
	def position(position, turned)
		cont = 0
		find = false
		while ((cont < @c.length)&&(!find))
			if (@c[cont].position == position)&&(@c[cont].turned == turned)
				find = true
			else
				cont+=1
			end
		end
		return cont
	end
	
	#candidate:: A TriangularPrismPosition objet
	#
	#returns:: true if candidate is in TriangularPrismShape
	def esta (candidate)
		cont = 0
		find = false
		while ((cont < @c.length)&&(!find))
			if (@c[cont].position == candidate.position)&&(@c[cont].turned == candidate.turned)
				find = true
			end
			cont+=1
		end
		return find
	end
	
	#position:: an OrderedPoint
	#
	#returns:: true if we have a triangular prism in position
	def esta_pos (position)
		cont = 0
		find = false
		while ((cont < @c.length)&&(!find))
			if (@c[cont].position == position)
				find = true
			end
			cont+=1
		end
		return find
	end
	
	def actual_occupation
		if @occupation1
			@occupation1.clear
		else
			@occupation1 = Array.new
		end
		@c.each {|shape|
			position = shape.position.clone
			position.z = 0
			if !@occupation1.include? position
				@occupation1.push position
			end
		}
		
	end
	
	def occupation (shape1)
		solution = false
		position = shape1.position.clone
		position.z = 0
		cont = @occupation1.length
		if !@occupation1
			@occupation1 = Array.new
			cont += 1
		elsif !@occupation1.include? position
			cont += 1
		end
		area = ((Constants::SIZE*Constants::HEIGHT_T)/2)*cont
		area_solar_percent = (Constants::SIZE_P*Constants::SIZE_P*Constants::OCCUPATION)/100
		if area <= area_solar_percent
			solution = true
		end
		return solution
	end
	
	#path:: path to save the shape in
	#
	#Saves the shape in the specified path
	def save(path)
		File.open(path, 'w') do |f|
			f.write("Type: #{@type}\n")
			@c.each{|shape|
				f.write("S: #{shape.position.x.to_f.to_m} #{shape.position.y.to_f.to_m} #{shape.position.z.to_f.to_m} #{shape.turned}\n")
			}
		end
	end
	
	#path:: path to load the shape from
	#
	#Loads the shape in the specified path
	def load(path)
		current = self.duplicate
		self.erase
		salir = false
		File.open(path, 'r') do |f|
			while (line = f.gets)&&(!salir)
				line_a1 = line.split(":")
				if line_a1[0].include? "Type"
					@type = line_a1[1]
				elsif line_a1[0] == "S"
					line_a = line_a1[1].split
					point = OrderedPoint.new(Geom::Point3d.new(line_a[0].to_f.m, line_a[1].to_f.m, line_a[2].to_f.m))
					if line_a[3].include? "true"
						turned = true
					else
						turned = false
					end
					prism = TriangularPrism.new(Constants::SIZE)
					shape = PositionTriangularPrism.new(prism, point, turned)
					@c.push(shape)
				else
					salir = true
					@c = current.c
					UI.messagebox("El archivo no tiene la estructura interna deseada")
				end
			end
		end
		self.repaint
	end
	
	#current:: a TriangularPrismShape object
	#
	#returns:: true if this TriangularPrismShape is equal to current
	def repe(current)
		igual = true
		cont = 0
		if current.c.length == @c.length
			while (cont < current.c.length)&&igual
				if !esta(current.c[cont])
					igual = false
				end
				cont+=1
			end
		else
			igual = false
		end
		return igual
	end
end


#This class represents the design when it's composed by hexagonal prisms
class HexagonalPrismShape
	#Array, for each prism I have 
	attr_accessor :c
	
	#The type of the shape: always hexagonal
	attr_accessor :type
	
	attr_accessor :occupation1
	
	#Array with all the free positions
	attr_reader :point
	#Array with all the free face positions
	attr_reader :face
	#Array with all the free turnedface positions
	attr_reader :turnedface
	#Array with all the free edge positions
	attr_reader :edge
	#Array with all the free turnededge positions
	attr_reader :turnededge
	
	#prism:: An HexagonalPrism object
	#position:: an OrderedPoint
	def initialize(prism, position)
		@c = Array.new
		@c.push(PositionHexagonalPrism.new(prism, position))
		@turnedface = [OrderedPoint.new(Geom::Point3d.new(prism.size+prism.size/2,-Constants::APOTEMA,0)), OrderedPoint.new(Geom::Point3d.new(prism.size+prism.size/2,Constants::APOTEMA,0)), OrderedPoint.new(Geom::Point3d.new(-prism.size-prism.size/2,-Constants::APOTEMA,0)), OrderedPoint.new(Geom::Point3d.new(-prism.size-prism.size/2,Constants::APOTEMA,0)), OrderedPoint.new(Geom::Point3d.new(0,-2*Constants::APOTEMA,0)), OrderedPoint.new(Geom::Point3d.new(0,2*Constants::APOTEMA,0))]
		@turnedface.sort
		@face = [OrderedPoint.new(Geom::Point3d.new(0,0,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,0,-Constants::HEIGHT_S))]
		@face.sort
		@edge =  [OrderedPoint.new(Geom::Point3d.new(prism.size+prism.size/2,Constants::APOTEMA,-Constants::HEIGHT_S)),OrderedPoint.new(Geom::Point3d.new(prism.size+prism.size/2,Constants::APOTEMA,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(prism.size+prism.size/2,-Constants::APOTEMA,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(prism.size+prism.size/2,-Constants::APOTEMA,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size-prism.size/2,-Constants::APOTEMA,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size-prism.size/2,-Constants::APOTEMA,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size-prism.size/2,Constants::APOTEMA,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size-prism.size/2,Constants::APOTEMA,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,2*Constants::APOTEMA,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,2*Constants::APOTEMA,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,-2*Constants::APOTEMA,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(0,-2*Constants::APOTEMA,-Constants::HEIGHT_S))]
		@edge.sort
		@turnededge = [OrderedPoint.new(Geom::Point3d.new(2*prism.size,0,0)), OrderedPoint.new(Geom::Point3d.new(-2*prism.size,0,0)), OrderedPoint.new(Geom::Point3d.new(-prism.size,2*Constants::APOTEMA,0)), OrderedPoint.new(Geom::Point3d.new(prism.size,2*Constants::APOTEMA,0)), OrderedPoint.new(Geom::Point3d.new(prism.size,-2*Constants::APOTEMA,0)), OrderedPoint.new(Geom::Point3d.new(-prism.size,-2*Constants::APOTEMA,0))]
		@turnededge.sort
		@point = [OrderedPoint.new(Geom::Point3d.new(2*prism.size,0,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(2*prism.size,0,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-2*prism.size,0,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-2*prism.size,0,-Constants::HEIGHT_S)),OrderedPoint.new(Geom::Point3d.new(-prism.size,2*Constants::APOTEMA,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size,2*Constants::APOTEMA,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(prism.size,2*Constants::APOTEMA,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(prism.size,2*Constants::APOTEMA,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(prism.size,-2*Constants::APOTEMA,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(prism.size,-2*Constants::APOTEMA,-Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size,-2*Constants::APOTEMA,Constants::HEIGHT_S)), OrderedPoint.new(Geom::Point3d.new(-prism.size,-2*Constants::APOTEMA,-Constants::HEIGHT_S))]
		@point.sort
		@type = "hexagonal"
	end
	
	#Repaints the view of the shape
	def repaint
		@c.each { |prism_shape|
			if !prism_shape.prism.group
				transfor = Constants::AXIOM_T
				t_position = Geom::Transformation.new(Geom::Point3d.new(prism_shape.position.x, prism_shape.position.y, prism_shape.position.z))
				transfor = transfor * t_position
				prism_shape.prism.paint(transfor)
			end
		}
	end
	
	#Deletes the current shape
	def erase
		@c.each { |prism_shape|
			prism_shape.erase
		}
		@c.clear
	end
	
	#returns:: a HexagonalPrismShape object identical to this
	def duplicate
		shape = HexagonalPrismShape.new(@c[0].prism, @c[0].position)
		cont = 1
		while cont<@c.length
			shape.c.push(PositionHexagonalPrism.new(@c[cont].prism.duplicate, @c[cont].position))
			cont+=1
		end
		shape.type = @type
		return shape
	end
	
	#position:: an OrderedPoint
	#
	#returns:: the array position where there is a shape with the same position
	def position (position)
		cont = 0
		find = false
		while ((cont < @c.length)&&(!find))
			if @c[cont].position == position
				find = true
			else
				cont+=1
			end
		end
		return cont
	end
	
	#candidate:: A HexagonalPrismPosition object
	#
	#returns:: true if candidate is in HexagonalPrismShape
	def esta (candidate)
		cont = 0
		find = false
		while ((cont < @c.length)&&(!find))
			if @c[cont].position == candidate.position
				find = true
			end
			cont+=1
		end
		return find
	end
	
	#position:: an OrderedPoint
	#
	#returns:: true if we have a hexagonal prism in position
	def esta_pos (position)
		cont = 0
		find = false
		while ((cont < @c.length)&&(!find))
			if @c[cont].position == position
				find = true
			end
			cont+=1
		end
		return find
	end
	
	def actual_occupation
		if @occupation1
			@occupation1.clear
		else
			@occupation1 = Array.new
		end
		@c.each {|shape|
			position = shape.position.clone
			position.z = 0
			if !@occupation1.include? position
				@occupation1.push position
			end
		}
		
	end
	
	def occupation (shape1)
		solution = false
		position = shape1.position.clone
		position.z = 0
		cont = @occupation1.length
		if !@occupation1
			@occupation1 = Array.new
			cont += 1
		elsif !@occupation1.include? position
			cont += 1
		end
		area = (((6*Constants::SIZE_H)*Constants::APOTEMA)/2)*cont
		area_solar_percent = (Constants::SIZE_P*Constants::SIZE_P*Constants::OCCUPATION)/100
		if area <= area_solar_percent
			solution = true
		end
		return solution
	end
	
	#path:: path to save the shape in
	#
	#Saves the shape in the specified path
	def save(path)
		File.open(path, 'w') do |f|
			f.write("Type: #{@type}\n")
			@c.each{|shape|
				f.write("S: #{shape.position.x.to_f.to_m} #{shape.position.y.to_f.to_m} #{shape.position.z.to_f.to_m}\n")
			}
		end
	end
	
	#path:: path to load the shape from
	#
	#Loads the shape in the specified path
	def load(path)
		current = self.duplicate
		self.erase
		salir = false
		File.open(path, 'r') do |f|
			while (line = f.gets)&&(!salir)
				line_a1 = line.split(":")
				if line_a1[0].include? "Type"
					@type = line_a1[1]
				elsif line_a1[0] == 'S'
					line_a = line_a1[1].split
					point = OrderedPoint.new(Geom::Point3d.new(line_a[0].to_f.m, line_a[1].to_f.m, line_a[2].to_f.m))
					prism = HexagonalPrism.new(Constants::SIZE_H)
					shape = PositionHexagonalPrism.new(prism, point)
					@c.push(shape)
				else
					salir = true
					@c = current.c
					UI.messagebox("El archivo no tiene la estructura interna deseada")
				end
			end
		end
		self.repaint
	end
	
	#current:: a HexagonalPrismShape object
	#
	#returns:: true if this HexagonalPrismShape is equal to current
	def repe(current)
		igual = true
		cont = 0
		if current.c.length == @c.length
			while (cont < current.c.length)&&igual
				if !esta(current.c[cont])
					igual = false
				end
				cont+=1
			end
		else
			igual = false
		end
		return igual
	end
end

#This class represents the contour line map
class Hill
	#Hash of array of points. There's one array for each hill.
	attr_accessor :points
	
	attr_accessor :group1
	
	attr_accessor :group2
	
	attr_accessor :z_d1
	
	attr_accessor :z_d2
	
	#Indicates if the contour line map is painted or not.
	attr_accessor :painted
	
	#Array with the tree positions
	attr_accessor :trees
	
	#Array of array with the points of the lakes
	attr_accessor :lakes
	
	#Initializes the hills
	def initialize
		@points = Hash.new(nil)
		@points["c1"] = Array.new
		@points["c2"] = Array.new
		@points_z = Hash.new(nil)
		@points_z2 = Hash.new(nil)
		@group1 = nil
		@group2 = nil
		@painted = false
		@trees = Array.new
		@trees_points = Array.new
		#Hash when we save the instances of each tree
		@instance_trees = Hash.new
		#Array of array with the points of the lakes
		@lakes = Array.new
		#Array of lakes groups
		@groups_lake = Array.new
	end
	
	#path:: path to load the contour line map from
	#
	#Loads the contour line map in the specified path
	def load(path)
		if @group1 != nil && @group2 != nil
			self.erase
		end
		salir = false
		File.open(path, 'r') do |f|
			z = 0
			while (line = f.gets) && (!salir)
				line_a1 = line.split(":")
				if line_a1[0].include? "Height"
					z = line_a1[1].to_f.m
					@points_z[z] = Array.new
					@points_z2[z] = Array.new
				elsif line_a1[0].include? "C1"
					line_a = line_a1[1].split(",")
					for i in 0..line_a.length-1
						line_point = line_a[i].split
						@points["c1"].push([line_point[0].to_f.m, line_point[1].to_f.m, z])
						@points_z[z].push([line_point[0].to_f.m, line_point[1].to_f.m, z])
					end
				elsif line_a1[0].include? "C2"
					line_a = line_a1[1].split(",")
					for i in 0..line_a.length-1
						line_point = line_a[i].split
						@points["c2"].push([line_point[0].to_f.m, line_point[1].to_f.m, z])
						@points_z2[z].push([line_point[0].to_f.m, line_point[1].to_f.m, z])
					end
				elsif line_a1[0].include? "Trees"
					line_a = line_a1[1].split(",")
					for i in 0..line_a.length-1
						line_point = line_a[i].split
						@trees.push([line_point[0].to_f.m, line_point[1].to_f.m, 0])
						array_aux = Array.new
						array_aux.push([line_point[0].to_f.m - 4.1.m, line_point[1].to_f.m - 3.1.m, 0])
						array_aux.push([line_point[0].to_f.m - 4.1.m, line_point[1].to_f.m + 3.1.m, 0])
						array_aux.push([line_point[0].to_f.m + 4.1.m, line_point[1].to_f.m - 3.1.m , 0])
						array_aux.push([line_point[0].to_f.m + 4.1.m, line_point[1].to_f.m + 3.1.m, 0])
						@trees_points.push array_aux
					end
				elsif line_a1[0].include? "Lakes"
					line_a = line_a1[1].split(",")
					if array_aux
						array_aux.clear
					end
					array_aux = Array.new
					for i in 0..line_a.length-1
						line_point = line_a[i].split
						array_aux.push([line_point[0].to_f.m, line_point[1].to_f.m, 0])
					end
					@lakes.push array_aux
				else
					salir = true
					UI.messagebox("El archivo no tiene la estructura interna deseada")
					if @group1 != nil && @group2 != nil
						self.erase
					end
				end
			end
		end
		return salir
	end
	
	#Paints the contour line map
	def paint
		@painted = true
		z_values = Array.new
		@points_z.each_key {|z|
			if !@points_z[z].empty?
				z_values.push(z)
			end
		}
		z_values.sort! { |z1, z2|
			z1 <=> z2
		}
		@z_d1 = Hash.new
		for i in 0..z_values.length-1
			if i == 0
				@z_d1[z_values[i]]=z_values[i]
			else
				@z_d1[z_values[i]]=z_values[i]-z_values[i-1]
			end
		end
		model = Sketchup.active_model
		materials = model.materials
		#mud = materials.add "Mud"
		#mud.texture = "c:\\Program Files (x86)\\Google\\Google SketchUp 8\\Plugins\\Practica\\lib\\resources\\textures\\mud.jpg"
		#mud.texture.size = 10.m
		@group1 = Sketchup.active_model.entities.add_group
		@points_z.each_key {|z|
			if !@points_z[z].empty?
				base = @group1.entities.add_face @points_z[z]
				base.pushpull @z_d1[z]
				#cont = 0
				#while cont < @group1.entities.length
				#	if @group1.entities[cont].typename == "Face"
				#		@group1.entities[cont].material = mud
				#	end
				#	cont+= 1
				#end
			end
		}
		transfor = Constants::HILL_T
		t_position = Geom::Transformation.new(Geom::Point3d.new(0, 0, 0))
		transfor = transfor * t_position
		@group1.transformation = transfor
			
			
		z_values1 = Array.new
		@points_z2.each_key {|z|
			if !@points_z2[z].empty?
				z_values1.push(z)
			end
		}
		z_values1.sort! { |z1, z2|
			z1 <=> z2
		}
		@z_d2 = Hash.new
		for i in 0..z_values1.length-1
			if i == 0
				@z_d2[z_values1[i]]=z_values1[i]
			else
				@z_d2[z_values1[i]]=z_values1[i]-z_values1[i-1]
			end
		end
		@group2 = Sketchup.active_model.entities.add_group
		@points_z2.each_key {|z|
			if !@points_z2[z].empty?
				base = @group2.entities.add_face @points_z2[z]
				base.pushpull @z_d2[z]
				#cont = 0
				#while cont < @group2.entities.length
				#	if @group2.entities[cont].typename == "Face"
				#		@group2.entities[cont].material = mud
				#	end
				#	cont+= 1
				#end
			end
		}
		transfor = Constants::HILL_T
		t_position = Geom::Transformation.new(Geom::Point3d.new(0, 0, 0))
		transfor = transfor * t_position
		@group2.transformation = transfor
		
		#Now we're going to paint a tree
		# First, load our pants (heh). This time we'll search for
		# the file inside our Components directory.
		pants_path = Sketchup.find_support_file "Practica/lib/resources/textures/finihed tree.skp" ,"Plugins" 
		pants_def = Sketchup.active_model.definitions.load pants_path

		@trees.each{|point|
			# Then define a location, and place our pants there.
			pants_location = Geom::Point3d.new point
			transform = Geom::Transformation.new pants_location
			transfor = Constants::HILL_T
			transform = transform * transfor
			entities = Sketchup.active_model.active_entities
			@instance_trees[point] = entities.add_instance pants_def, transform
		}
		
		#Now we're going to paint the lakes
		cont = 0
		transfor = Constants::HILL_T
		t_position = Geom::Transformation.new(Geom::Point3d.new(0, 0, 0))
		transfor = transfor * t_position
		m = materials.add "Water"
		pants_path = Sketchup.find_support_file "Practica/lib/resources/textures/agua.jpg" ,"Plugins" 
		m.texture = pants_path
		m.texture.size = 600
		while (cont < @lakes.length)
			@groups_lake[cont] = Sketchup.active_model.entities.add_group
			face = @groups_lake[cont].entities.add_face @lakes[cont]
			face.pushpull 0.3
			@groups_lake[cont].transformation = transfor
			face.material = m
			cont+=1
		end

	end
	
	#candidate:: It can be a CubePosition/TriangularPrismPosition/HexagonalPrismPosition object
	#
	#returns:: true if candidate intersect with the contour line map
	def intersect(candidate)
		salir = false
		transfor = Constants::AXIOM_T
		t_position = Geom::Transformation.new(Geom::Point3d.new(candidate.position.x, candidate.position.y, candidate.position.z))
		tr1 = transfor * t_position
		if candidate.type.include? "cube"
			candidate.cube.paint(tr1)
			tr = Geom::Transformation.new()
			position =  Geom::Transformation.new(Geom::Point3d.new(candidate.position.x,candidate.position.y,candidate.position.z))
			tr2 = position*Constants::INTERSECTION
		else
			if candidate.type.include? "triangle"
				transfor = Constants::AXIOM_T
				t_position = Geom::Transformation.new
				t_position = [candidate.position.x, candidate.position.y, candidate.position.z]
				transfor = transfor * t_position
				candidate.prism.paint(transfor)
				if (candidate.turned)
					vector = Geom::Vector3d.new(1,0,0)
					transformation = Geom::Transformation.rotation transfor, vector, Math::PI
					candidate.prism.group.transform! transformation
				end
			else
				candidate.prism.paint(tr1)
			end
			tr = Geom::Transformation.new()
			position =  Geom::Transformation.new(Geom::Point3d.new(candidate.position.x,candidate.position.y,candidate.position.z))
			point_aux = Geom::Point3d.new (Constants::VECTOR_INTERSECTION_HILL[1])
			trans = point_aux.transform position
			tr2 = position*Constants::INTERSECTION
			if candidate.type.include? "triangle"
				if candidate.turned
					vector = Geom::Vector3d.new(1,0,0)
					tr2 = Geom::Transformation.rotation trans, vector, Math::PI
				end
			end
		end
		#We ckeck if our shape is intersected with the first hill
		z_value = -1
		cont = 0
		position = Array.new
		if candidate.type.include? "cube"
			while cont < candidate.cube.group.entities.length
				if candidate.cube.group.entities[cont].typename == "Edge"
					position.push(candidate.cube.group.entities[cont].vertices[0].position) if !position.include?(candidate.cube.group.entities[cont].vertices[0].position) && candidate.cube.group.entities[cont].vertices[0].position.z == 0
					position.push(candidate.cube.group.entities[cont].vertices[1].position) if !position.include?(candidate.cube.group.entities[cont].vertices[1].position) && candidate.cube.group.entities[cont].vertices[1].position.z == 0
				end
				cont += 1
			end
		else
			while cont < candidate.prism.group.entities.length
				if candidate.prism.group.entities[cont].typename == "Edge"
					position.push(candidate.prism.group.entities[cont].vertices[0].position) if !position.include?(candidate.prism.group.entities[cont].vertices[0].position) && candidate.prism.group.entities[cont].vertices[0].position.z == 0
					position.push(candidate.prism.group.entities[cont].vertices[1].position) if !position.include?(candidate.prism.group.entities[cont].vertices[0].position) && candidate.prism.group.entities[cont].vertices[1].position.z == 0
				end
				cont += 1
			end
		end
		cont = 0
		while cont < position.length
			if candidate.type.include? "triangle"
				if candidate.turned
					prueba =  Geom::Transformation.new trans
					position[cont].transform! prueba
				end
			end
			position[cont].transform! tr2
			cont+=1
		end
		if candidate.type.include? "triangle"
			if candidate.turned
				position1 = OrderedPoint.new(Geom::Point3d.new Constants::VECTOR_INTERSECTION_HILL[1])
			else
				position1 = OrderedPoint.new(Geom::Point3d.new Constants::VECTOR_INTERSECTION_HILL[0])
			end
		else
			position1 = OrderedPoint.new(Geom::Point3d.new Constants::VECTOR_INTERSECTION_HILL[0])
		end
		position1 = position1 + candidate.position
		@z_d1.each_key{|z|
			if (position1.z < z)&&(position1.z >= z - @z_d1[z])
				z_value = z
			end
		}
		cont = 0
		if z_value != -1
			while (cont < position.length)&&!salir
				j = @points_z[z_value].length - 1
				i = 0
				while (i <@points_z[z_value].length)
					array = @points_z[z_value]
					if ( ((array[i][1]>position[cont].y) != (array[j][1]>position[cont].y)) && (position[cont].x< (array[j][0]-array[i][0]) * (position[cont].y-array[i][1]) / (array[j][1]-array[i][1]) + array[i][0]) )
						salir = !salir
					end
					j = i
					i += 1
				end
				cont+=1
			end
		end
		if !salir
			#We ckeck if our shape is intersected with the other hill
			z_value = -1
			@z_d2.each_key{|z|
				if (position1.z < z)&&(position1.z >= z - @z_d2[z])
					z_value = z
				end
			}
			if z_value != -1
				cont = 0
				while (cont < position.length)&&!salir
					j = @points_z2[z_value].length - 1
					i = 0
					while (i <@points_z2[z_value].length)
						array = @points_z2[z_value]
						if ( ((array[i][1]>position[cont].y) != (array[j][1]>position[cont].y)) && (position[cont].x< (array[j][0]-array[i][0]) * (position[cont].y-array[i][1]) / (array[j][1]-array[i][1]) + array[i][0]) )
							salir = !salir
						end
						j = i
						i += 1
					end
					cont+=1
				end
			end 
		end
		if !salir && !@trees_points.empty?
			#we teste if our shape is intersected with a tree
			cont = 0
			while (cont < position.length)&&!salir
				n = 0
				while (n < @trees_points.length)&&!salir
					j = @trees_points[n].length - 1
					i = 0
					while (i <@trees_points[n].length)
						array = @trees_points[n]
						if ( ((array[i][1]>position[cont].y) != (array[j][1]>position[cont].y)) && (position[cont].x< (array[j][0]-array[i][0]) * (position[cont].y-array[i][1]) / (array[j][1]-array[i][1]) + array[i][0]) )
							salir = !salir
						end
						j = i
						i += 1
					end
					n+=1
				end
				cont+=1
			end
		end
		if !salir&&!@lakes.empty?
			#we teste if our shape is intersected with a lake
			cont = 0
			while (cont < position.length)&&!salir
				n = 0
				while (n < @lakes.length)&&!salir
					j = @lakes[n].length - 1
					i = 0
					while (i <@lakes[n].length)
						array = @lakes[n]
						if ( ((array[i][1]>position[cont].y) != (array[j][1]>position[cont].y)) && (position[cont].x< (array[j][0]-array[i][0]) * (position[cont].y-array[i][1]) / (array[j][1]-array[i][1]) + array[i][0]) )
							salir = !salir
						end
						j = i
						i += 1
					end
					n+=1
				end
				cont+=1
			end
		end
		candidate.erase
		return salir
	end
	
	#Deletes the contour line map
	def erase
		@painted = false
		Sketchup.active_model.entities.erase_entities @group1
		Sketchup.active_model.entities.erase_entities @group2
		@instance_trees.each_key {|point|
			Sketchup.active_model.active_entities.erase_entities @instance_trees[point]
		}
		@instance_trees.clear
		@instance_trees = Hash.new(nil)
		@trees_points.clear
		@trees_points = Array.new
		@trees.clear
		@trees = Array.new
		@group1 = nil
		@group2= nil
		@points.clear
		@points_z.clear
		@points_z2.clear
		@z_d1.clear
		@z_d2.clear
		@lakes.clear
		@lakes = Array.new
		@groups_lake.each{|group|
			Sketchup.active_model.entities.erase_entities group
		}
		@groups_lake.clear
		@groups_lake = Array.new
		@points = Hash.new(nil)
		@points["c1"] = Array.new
		@points["c2"] = Array.new
		@points_z = Hash.new(nil)
		@points_z2 = Hash.new(nil)
	end
end

#This class represents the pyramid where the design must be inside
class Pyramid
	#The SU objects that groups the units
	attr_accessor :group
	
	#This variable indicates us if the pyramide is painted
	attr_accessor :painted
	
	#Initializes Pyramid
	def initialize
		@group = nil
		@painted = false
	end
	
	#Paints the pyramid whose base is 100 metres by 100 metres and whose angle is 30 degrees.
	def paint
		if !@painted
			@painted = true
			@group = Sketchup.active_model.entities.add_group
			@group.entities.add_line Constants::PTS_V[0], Constants::VECTOR_HEIGHT_PYRAMIDE[0] 
			@group.entities.add_line Constants::PTS_V[1], Constants::VECTOR_HEIGHT_PYRAMIDE[0] 
			@group.entities.add_line Constants::PTS_V1[0], Constants::VECTOR_HEIGHT_PYRAMIDE[0] 
			@group.entities.add_line Constants::PTS_V1[1], Constants::VECTOR_HEIGHT_PYRAMIDE[0] 
		end
	end
	
	#Deletes the pyramid.
	def erase
		@painted = false
		Sketchup.active_model.entities.erase_entities @group
	end
end