require 'sketchup' # First we pull in the standard API hooks.
require 'Practica//lib//structures.rb'
require 'Practica//lib//utils.rb'

#This class represents a rule
class Rules
	#Internal rule id
	attr_accessor :rule_id
	
	#We must work with a shape
	attr_accessor :shape
	
	#This variable indicates us how it's the transformation
	attr_accessor :type_transformation
	
	attr_accessor :shape_l
	attr_accessor :shape_rule
	
	attr_accessor :line_group
	attr_accessor :group_arrow
	
	#rule_id:: internal id for the rule
	#This method initializes the rule
	def initialize(rule_id, shape, type_transformation, group_arrow, line_group, shape_l, shape_rule)
		@rule_id = rule_id
		@shape = shape
		@type_transformation = type_transformation
		@group_arrow = group_arrow
		@line_group = line_group
		@shape_l = shape_l
		@shape_rule = shape_rule
	end
	
	#execution:: a Execution object
	#
	#Paints the rule
	def paint(execution)
		if execution.grammar
			idx = execution.grammar.rules.size 
		else
			idx = 0
		end
		t = idx

		# Transform the layout transformation of the shapes
		t_left = Geom::Transformation.new
		t_right = Geom::Transformation.new
		t.times do
			t_left = Constants::DESCEND_T * t_left
			t_right = Constants::DESCEND_T * t_right
		end
		t_left = Constants::LEFT_T * t_left
		t_right = Constants::RIGHT_T * t_right
		
		@shape.paint(t_left)
		
		if @shape.type.include? "cube"
			if @type_transformation == "face"
				t_rule = Geom::Transformation.new
				t_rule = [@shape.size,0,0]
			elsif @type_transformation == "point"
				t_rule = Geom::Transformation.new
				t_rule = [@shape.size,@shape.size,-Constants::HEIGHT_S]
			elsif @type_transformation == "edge"
				t_rule = Geom::Transformation.new
				t_rule = [@shape.size,@shape.size,0]
			end
		elsif @shape.type.include? "triangle"
			if type_transformation == "face2"
				t_rule = Geom::Transformation.new
				t_rule = [Constants::SIZE/2,Constants::HEIGHT_T,-Constants::HEIGHT_S] 
			elsif type_transformation == "face"
				t_rule = Geom::Transformation.new
				t_rule = [0,0,-prism.size]
			elsif type_transformation == "edge2"
				t_rule = Geom::Transformation.new
				t_rule = [Constants::SIZE/2,Constants::HEIGHT_T,-2*Constants::HEIGHT_S] 
			elsif type_transformation == "edge"
				t_rule = Geom::Transformation.new
				t_rule = [prism.size,0,0]
			elsif type_transformation == "edge3"
				t_rule = Geom::Transformation.new
				t_rule = [prism.size,0,-Constants::HEIGHT_S]
			elsif type_transformation == "point2"
				t_rule = Geom::Transformation.new
				t_rule = [Constants::SIZE,0,-2*Constants::HEIGHT_S] 
			elsif type_transformation == "point"
				t_rule = Geom::Transformation.new
				t_rule = [Constants::SIZE,0,-Constants::HEIGHT_S] 
			end
		end
		
		# Draw an arrow, for the rule
		point1 = Constants::PTS_ARROW[0].clone
		point2 = Constants::PTS_ARROW[1].clone
		point3 = Constants::PTS_ARROW[2].clone
		point4 = Constants::PTS_ARROW[3].clone
				
		t.times do
			point1.transform! Constants::DESCEND_T 
			point2.transform! Constants::DESCEND_T 
			point3.transform! Constants::DESCEND_T 
			point4.transform! Constants::DESCEND_T 
		end

		@group_arrow = Sketchup.active_model.entities.add_group
		@group_arrow.entities.add_line point1, point2
		@group_arrow.entities.add_line point3, point2
		@group_arrow.entities.add_line point4, point2
		
		@shape_l = @shape.duplicate
		@shape_l.paint(t_right)
		t_rule_final = t_right* t_rule
		@shape_rule = @shape_l.duplicate
		if @shape.type.include? "cube"
			@shape_rule.paint(t_rule_final)
		elsif shape.type.include? "triangle"
			@shape_rule.paint(t_rule_final)
			if ((@type_transformation == "face2")||(@type_transformation == "edge2")||(@type_transformation == "point2")||(@type_transformation == "edge3"))
				vector = Geom::Vector3d.new(1,0,0)
				transformation = Geom::Transformation.rotation t_rule_final, vector, Math::PI
				@shape_rule.group.transform! transformation
			end
			
		end
		point_h1 = Constants::PTS_H[0].clone
		point_h2 = Constants::PTS_H[1].clone
				
		t.times do
			point_h1.transform! Constants::DESCEND_T 
			point_h2.transform! Constants::DESCEND_T 
		end
		@line_group = Sketchup.active_model.entities.add_group
		@line_group.entities.add_line point_h1,point_h2
		
		#We block every variables
		@shape.group.blocked = true
		@group_arrow.blocked = true
		@line_group.blocked = true
		@shape_l.group.blocked = true
		@shape_rule.group.blocked = true
		
	end
	
	#Deletes the rule
	def erase()

		if @group_arrow
			@group_arrow.locked = false
			Sketchup.active_model.entities.erase_entities @group_arrow
		end

		if @line_group
			@line_group.locked = false
			Sketchup.active_model.entities.erase_entities @line_group
		end
		
		@shape.group.locked = false
		@shape.erase
		@shape_l.group.locked = false
		@shape_l.erase
		@shape_rule.group.locked = false
		@shape_rule.erase
	end
	
	#returns:: a Rule object identical to this
	def duplicate
		return Rules.new(@rule_id, @shape, @type_transformation, @group_arrow, @line_group, @shape_l, @shape_rule)
	end
	
end


#Abstract representation of a grammar, that is, a set of rules
class Grammar
	#Rules array
	attr_accessor :rules
	
	#Represents the design
	attr_accessor :axiom
	
	#True if the grammar has been saved
	attr_accessor :saved
	
	#Initializing
	def initialize()
		@rules = Array.new
		@saved = true
	end
	
	#rule:: a Rule object to add
	#
	#Adds a rule to the grammar
	def add_rule(rule)
		@rules.push(rule)
	end
	
	#i:: index of the rule to remove
	#
	#Removes the rule in the grammar with the index i
	def remove_rule(i)
		#Delete from the array
		@rules.delete_at(i)
	end
	
	#i:: index of the rule to remove
	#
	#Removes and deletes the rule in the grammar
	def delete_rule(i)
		rule = @rules[i]
		rule.erase
		#Delete from the array
		@rules.delete_at(i)
	end
	
	#shape:: shape that will be the axiom of the grammar
	#
	#Sets the axiom to the specified shape
	def axiom=(shape)
		@axiom = shape
	end	
	
	#rule:: a Rule object, representing one of the grammar rules
	#
	#returns:: the index of the rule (not the ID)
	def get_rule_index(rule)
		result = nil
		i = 0
		while ((i < @rules.size) && !result)
			if @rules[i].rule_id == rule.rule_id
				result = i
			end
			i+=1
		end
		return result
	end
	
	#rule_id:: an id
	#
	#returns the rule with the internal id rule_id. In case it does not exist, returns nil
	def search_rule_by_id(rule_id)
		i = 0
		found = false
		size = @rules.size
		rule = nil
		
		while i < size and !found
			if @rules[i].rule_id == rule_id
				rule = @rules[i]
				found = true
			end
			i +=1
		end
		return rule
	end
	
	#type_transoformation
	#
	#returns the rule with the internal id rule_id. In case it does not exist, returns nil
	def search_rule_by_transformation(type_transformation)
		i = 0
		found = false
		size = @rules.size
		rule = nil
		
		while i < size and !found
			if @rules[i].type_transformation == type_transformation
				found = true
			end
			i +=1
		end
		return found
	end
	
	#shape_id:: an id
	#
	#returns the shape with the internal id shape_id. In case it does not exist, returns nil
	def search_shape_by_id(shape_id)
		i = 0
		found = false
		size = @rules.size
		shape = nil
		
		while i < size and !found
			if @rules[i].left.shape_id == shape_id
				shape = @rules[i].left
				found = true
			elsif @rules[i].right.shape_id == shape_id
				shape = @rules[i].right
				found = true
			end
			i +=1
		end
		return shape
	end
	
	#returns:: a Grammar object identical to this
	def duplicate
		grammar = Grammar.new
		cont = 0
		while (cont<@rules.length)
			grammar.rules.push(@rules[cont].duplicate)
			cont+=1
		end
		return grammar
	end
end

#This class gathers an Execution object and the necessary information in order to save it in a file
class Execution
	#A Grammar object
	attr_accessor :grammar
	
	#The current design
	attr_accessor :current_shape
	
	#This variable indicates if the area constraint is activated
	attr_accessor :area
	
	attr_accessor :volumen
	
	#This variable indicates if the number of shapes constraint is activated
	attr_accessor :num_shapes
	
	#Maximum number of shapes
	attr_accessor :max_num_shapes
	
	#An array of pairs [rule_id, transformation] applied to the axiom, in order of application
	attr_accessor :execution_history
	
	#This variable helps us to undo the last step
	attr_accessor :num_steps
	
	#The contour line map
	attr_accessor :hill
	
	#This variable indicates if the volume constraint is activated
	attr_accessor :pyramid
	
	#Flag wich indicates us if we are goint to apply horizontal rules
	attr_accessor :left
	
	#Flag wich indicates us if we are goint to apply up/down rules
	attr_accessor :up
	
	#Feedback. This variable show the constranints wich are activated
	attr_accessor :text
	
	#Flag that indicates us if we have applied a rule or not
	attr_accessor :apply_r
	
	#The design
	attr_accessor :axiom
	
	#grammar:: the grammar to apply
	#current_shape:: shape in which to store the design
	def initialize(grammar, current_shape)
		@grammar= grammar
		@current_shape = current_shape
		@area = true
		@volumen = false
		@execution_history = Array.new
		@num_steps = Array.new
		@hill = Hill.new
		@pyramid = Pyramid.new
		@num_shapes = false
		@up = false
		@left = false
		@max_num_shapes = Constants::NUM_SHAPES
		@text = Sketchup.active_model.add_note "", 0.007, 0.95
		@apply_r = false
	end
	
	#rule_id:: internal id of the rule to apply to the current shape
	#current_shape:: the current design
	#backtraking:: true if we're using a backtraking algorithm
	#
	#Applies the specified rule to the current shape.
	def apply_rule(rule_id, current_shape, backtraking)
		rule = @grammar.search_rule_by_id(rule_id)
		random = rand(current_shape.c.length)
		current_shape.c.sort! { |shape1, shape2|
			shape1.position <=> shape2.position
		}
		if current_shape.type.include? "cube"
			if @up && @left
				rule = @grammar.search_rule_by_id(rule_id)
			elsif @up
				if rule.type_transformation == "face"
					rule.type_transformation = "faceup"
				elsif rule.type_transformation == "edge"
					rule.type_transformation = "edgeup"
				end
			elsif @left
				if rule.type_transformation == "face"
					rule.type_transformation = "faceleft"
				elsif rule.type_transformation == "edge"
					rule.type_transformation = "edgeleft"
				end
			end
		end
		if rule.type_transformation == "point"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.point.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.point.length))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.point[cont].x
					y = new_point.y + current_shape.point[cont].y
					z = new_point.z + current_shape.point[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					if current_shape.type.include? "triangle"
						turned = current_shape.c[cont_shape].turned
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if current_shape.c[cont2].turned == turned
									free_2 = false
								else
									cont2+=1
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						x = new_point.x + current_shape.point[cont].x
						y = new_point.y + current_shape.point[cont].y
						z = new_point.z + current_shape.point[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						if current_shape.type.include? "triangle"
							turned = current_shape.c[cont_shape].turned
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if current_shape.c[cont2].turned ==turned
										free_2 = false
									else
										cont2+=1
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.point.length)
					cont = random_2
					while ((!free)&&(cont<current_shape.point.length))
						new_point = current_shape.c[cont_shape].position.clone
						x = new_point.x + current_shape.point[cont].x
						y = new_point.y + current_shape.point[cont].y
						z = new_point.z + current_shape.point[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						if current_shape.type.include? "triangle"
							turned = current_shape.c[cont_shape].turned
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if current_shape.c[cont2].turned == turned
										free_2 = false
									else
										cont2+=1
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							x = new_point.x + current_shape.point[cont].x
							y = new_point.y + current_shape.point[cont].y
							z = new_point.z + current_shape.point[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							if current_shape.type.include? "triangle"
								turned = current_shape.c[cont_shape].turned
							end
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									if current_shape.type.include? "triangle"
										if current_shape.c[cont2].turned == turned
											free_2 = false
										else
											cont2+=1
										end
									else
										free_2 = false
									end
								else
									cont2+=1
								end
							end
							if (free_2)
								free = true
							else
								cont+=1
							end
						end
						if (!free)
							cont_shape += 1
						end
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "pointup"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.pointup.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.pointup.length))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.pointup[cont].x
					y = new_point.y + current_shape.pointup[cont].y
					z = new_point.z + current_shape.pointup[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							free_2 = false
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						x = new_point.x + current_shape.pointup[cont].x
						y = new_point.y + current_shape.pointup[cont].y
						z = new_point.z + current_shape.pointup[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								free_2 = false
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.pointup.length)
					cont = random_2
					while ((!free)&&(cont<current_shape.pointup.length))
						new_point = current_shape.c[cont_shape].position.clone
						x = new_point.x + current_shape.pointup[cont].x
						y = new_point.y + current_shape.pointup[cont].y
						z = new_point.z + current_shape.pointup[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								free_2 = false
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							x = new_point.x + current_shape.pointup[cont].x
							y = new_point.y + current_shape.pointup[cont].y
							z = new_point.z + current_shape.pointup[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									free_2 = false
								else
									cont2+=1
								end
							end
							if (free_2)
								free = true
							else
								cont+=1
							end
						end
						if (!free)
							cont_shape += 1
						end
					end
				end
			end
			rule.type_transformation = "point"
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "pointleft"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.pointleft.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.pointoleft.length))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.pointleft[cont].x
					y = new_point.y + current_shape.pointleft[cont].y
					z = new_point.z + current_shape.pointleft[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							free_2 = false
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						x = new_point.x + current_shape.pointleft[cont].x
						y = new_point.y + current_shape.pointleft[cont].y
						z = new_point.z + current_shape.pointleft[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								free_2 = false
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.pointleft.length)
					cont = random_2
					while ((!free)&&(cont<current_shape.pointleft.length))
						new_point = current_shape.c[cont_shape].position.clone
						x = new_point.x + current_shape.pointleft[cont].x
						y = new_point.y + current_shape.pointleft[cont].y
						z = new_point.z + current_shape.pointleft[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								free_2 = false
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							x = new_point.x + current_shape.pointleft[cont].x
							y = new_point.y + current_shape.pointleft[cont].y
							z = new_point.z + current_shape.pointleft[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									free_2 = false
								else
									cont2+=1
								end
							end
							if (free_2)
								free = true
							else
								cont+=1
							end
						end
						if (!free)
							cont_shape += 1
						end
					end
				end
			end
			rule.type_transformation = "point"
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "point2"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.turnedpoint.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.turnedpoint.length))
					new_point = current_shape.c[cont_shape].position.clone
					if current_shape.type.include? "triangle"
						turned = !current_shape.c[cont_shape].turned
						if turned == false
							x = new_point.x - current_shape.turnedpoint[cont].x
							y = new_point.y - current_shape.turnedpoint[cont].y
							z = new_point.z - current_shape.turnedpoint[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						else
							x = new_point.x + current_shape.turnedpoint[cont].x
							y = new_point.y + current_shape.turnedpoint[cont].y
							z = new_point.z + current_shape.turnedpoint[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
					else
						x = new_point.x + current_shape.turnedpoint[cont].x
						y = new_point.y + current_shape.turnedpoint[cont].y
						z = new_point.z + current_shape.turnedpoint[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					end
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if turned
									if current_shape.c[cont2].turned == true
										free_2 = false
									else
										cont2+=1
									end
								else
									if current_shape.c[cont2].turned == false
										free_2 = false
									else
										cont2+=1
									end
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						if current_shape.type.include? "triangle"
							turned = !current_shape.c[random].turned
							if turned == false
								x = new_point.x - current_shape.turnedpoint[cont].x
								y = new_point.y - current_shape.turnedpoint[cont].y
								z = new_point.z - current_shape.turnedpoint[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							else
								x = new_point.x + current_shape.turnedpoint[cont].x
								y = new_point.y + current_shape.turnedpoint[cont].y
								z = new_point.z + current_shape.turnedpoint[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							end
						else
							x = new_point.x + current_shape.turnedpoint[cont].x
							y = new_point.y + current_shape.turnedpoint[cont].y
							z = new_point.z + current_shape.turnedpoint[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if turned
										if current_shape.c[cont2].turned == true
											free_2 = false
										else
											cont2+=1
										end
									else
										if current_shape.c[cont2].turned == false
											free_2 = false
										else
											cont2+=1
										end
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.turnedpoint.length)
					cont = random_2
					while ((!free)&&(cont<current_shape.turnedpoint.length))
						new_point = current_shape.c[cont_shape].position.clone
						if current_shape.type.include? "triangle"
							turned = !current_shape.c[cont_shape].turned
							if turned == false
								x = new_point.x - current_shape.turnedpoint[cont].x
								y = new_point.y - current_shape.turnedpoint[cont].y
								z = new_point.z - current_shape.turnedpoint[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							else
								x = new_point.x + current_shape.turnedpoint[cont].x
								y = new_point.y + current_shape.turnedpoint[cont].y
								z = new_point.z + current_shape.turnedpoint[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							end
						else
							x = new_point.x + current_shape.turnedpoint[cont].x
							y = new_point.y + current_shape.turnedpoint[cont].y
							z = new_point.z + current_shape.turnedpoint[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if turned
										if current_shape.c[cont2].turned == true
											free_2 = false
										else
											cont2+=1
										end
									else
										if current_shape.c[cont2].turned == false
											free_2 = false
										else
											cont2+=1
										end
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							if current_shape.type.include? "triangle"
								turned = !current_shape.c[random].turned
								if turned == false
									x = new_point.x - current_shape.turnedpoint[cont].x
									y = new_point.y - current_shape.turnedpoint[cont].y
									z = new_point.z - current_shape.turnedpoint[cont].z
									new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
								else
									x = new_point.x + current_shape.turnedpoint[cont].x
									y = new_point.y + current_shape.turnedpoint[cont].y
									z = new_point.z + current_shape.turnedpoint[cont].z
									new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
								end
							else
								x = new_point.x + current_shape.turnedpoint[cont].x
								y = new_point.y + current_shape.turnedpoint[cont].y
								z = new_point.z + current_shape.turnedpoint[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							end
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									if current_shape.type.include? "triangle"
										if turned
											if current_shape.c[cont2].turned == true
												free_2 = false
											else
												cont2+=1
											end
										else
											if current_shape.c[cont2].turned == false
												free_2 = false
											else
												cont2+=1
											end
										end
									else
										free_2 = false
									end
								else
									cont2+=1
								end
							end
							if (free_2)
								free = true
							else
								cont+=1
							end
						end
						if (!free)
							cont_shape += 1
						end
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "face2"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.turnedface.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.turnedface.length))
					new_point = current_shape.c[cont_shape].position.clone
					cont_final = cont_shape
					if current_shape.type.include? "triangle"
						turned = !current_shape.c[cont_shape].turned
						if turned == false
							x = new_point.x - current_shape.turnedface[cont].x
							y = new_point.y - current_shape.turnedface[cont].y
							z = new_point.z - current_shape.turnedface[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						else
							x = new_point.x + current_shape.turnedface[cont].x
							y = new_point.y + current_shape.turnedface[cont].y
							z = new_point.z + current_shape.turnedface[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
					else
						x = new_point.x + current_shape.turnedface[cont].x
						y = new_point.y + current_shape.turnedface[cont].y
						z = new_point.z + current_shape.turnedface[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if turned
									if current_shape.c[cont2].turned == true
										free_2 = false
									else
										cont2+=1
									end
								else
									if current_shape.c[cont2].turned == false
										free_2 = false
									else
										cont2+=1
									end
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						if current_shape.type.include? "triangle"
							turned = !current_shape.c[random].turned
							if turned == false
								x = new_point.x - current_shape.turnedface[cont].x
								y = new_point.y - current_shape.turnedface[cont].y
								z = new_point.z - current_shape.turnedface[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							else
								x = new_point.x + current_shape.turnedface[cont].x
								y = new_point.y + current_shape.turnedface[cont].y
								z = new_point.z + current_shape.turnedface[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							end
						else
							x = new_point.x + current_shape.turnedface[cont].x
							y = new_point.y + current_shape.turnedface[cont].y
							z = new_point.z + current_shape.turnedface[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if turned
										if current_shape.c[cont2].turned == true
											free_2 = false
										else
											cont2+=1
										end
									else
										if current_shape.c[cont2].turned == false
											free_2 = false
										else
											cont2+=1
										end
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.turnedface.length)
					cont = random_2
					while ((!free)&&(cont<current_shape.turnedface.length))
						new_point = current_shape.c[cont_shape].position.clone
						cont_final = cont_shape
						if current_shape.type.include? "triangle"
							turned = !current_shape.c[cont_shape].turned
							if turned == false
								x = new_point.x - current_shape.turnedface[cont].x
								y = new_point.y - current_shape.turnedface[cont].y
								z = new_point.z - current_shape.turnedface[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							else
								x = new_point.x + current_shape.turnedface[cont].x
								y = new_point.y + current_shape.turnedface[cont].y
								z = new_point.z + current_shape.turnedface[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							end
						else
							x = new_point.x + current_shape.turnedface[cont].x
							y = new_point.y + current_shape.turnedface[cont].y
							z = new_point.z + current_shape.turnedface[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if turned
										if current_shape.c[cont2].turned == true
											free_2 = false
										else
											cont2+=1
										end
									else
										if current_shape.c[cont2].turned == false
											free_2 = false
										else
											cont2+=1
										end
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							if current_shape.type.include? "triangle"
								turned = !current_shape.c[random].turned
								if turned == false
									x = new_point.x - current_shape.turnedface[cont].x
									y = new_point.y - current_shape.turnedface[cont].y
									z = new_point.z - current_shape.turnedface[cont].z
									new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
								else
									x = new_point.x + current_shape.turnedface[cont].x
									y = new_point.y + current_shape.turnedface[cont].y
									z = new_point.z + current_shape.turnedface[cont].z
									new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
								end
							else
								x = new_point.x + current_shape.turnedface[cont].x
								y = new_point.y + current_shape.turnedface[cont].y
								z = new_point.z + current_shape.turnedface[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							end
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									if current_shape.type.include? "triangle"
										if turned
											if current_shape.c[cont2].turned == true
												free_2 = false
											else
												cont2+=1
											end
										else
											if current_shape.c[cont2].turned == false
												free_2 = false
											else
												cont2+=1
											end
										end
									else
										free_2 = false
									end
								else
									cont2+=1
								end
							end
							if (free_2)
								free = true
							else
								cont+=1
							end
						end
						if (!free)
							cont_shape += 1
						end
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "face"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.face.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.face.length))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.face[cont].x
					y = new_point.y + current_shape.face[cont].y
					z = new_point.z + current_shape.face[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					if current_shape.type.include? "triangle"
						turned = current_shape.c[cont_shape].turned
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if current_shape.c[cont2].turned == turned
									free_2 = false
								else
									cont2+=1
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end

				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						x = new_point.x + current_shape.face[cont].x
						y = new_point.y + current_shape.face[cont].y
						z = new_point.z + current_shape.face[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						if current_shape.type.include? "triangle"
							turned = current_shape.c[cont_shape].turned
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if current_shape.c[cont2].turned == turned
										free_2 = false
									else
										cont2+=1
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end

						if (free_2)
							free = true
						else
							cont+=1
						end
					end

					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.face.length)
					cont = random_2
					while ((!free)&&(cont<current_shape.face.length))
						new_point = current_shape.c[cont_shape].position.clone
						x = new_point.x + current_shape.face[cont].x
						y = new_point.y + current_shape.face[cont].y
						z = new_point.z + current_shape.face[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						if current_shape.type.include? "triangle"
							turned = current_shape.c[cont_shape].turned
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if current_shape.c[cont2].turned == turned
										free_2 = false
									else
										cont2+=1
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end

						if (free_2)
							free = true
						else
							cont+=1
						end
					end

					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							x = new_point.x + current_shape.face[cont].x
							y = new_point.y + current_shape.face[cont].y
							z = new_point.z + current_shape.face[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							if current_shape.type.include? "triangle"
								turned = current_shape.c[cont_shape].turned
							end
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									if current_shape.type.include? "triangle"
										if current_shape.c[cont2].turned == turned
											free_2 = false
										else
											cont2+=1
										end
									else
										free_2 = false
									end
								else
									cont2+=1
								end
							end

							if (free_2)
								free = true
							else
								cont+=1
							end
						end

						if (!free)
							cont_shape += 1
						end
					end
				end

			end

			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "faceup"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.faceup.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.faceup.length))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.faceup[cont].x
					y = new_point.y + current_shape.faceup[cont].y
					z = new_point.z + current_shape.faceup[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							free_2 = false
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end

				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						x = new_point.x + current_shape.faceup[cont].x
						y = new_point.y + current_shape.faceup[cont].y
						z = new_point.z + current_shape.faceup[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								free_2 = false
							else
								cont2+=1
							end
						end

						if (free_2)
							free = true
						else
							cont+=1
						end
					end

					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.faceup.length)
					cont = random_2
					while ((!free)&&(cont<current_shape.faceup.length))
						new_point = current_shape.c[cont_shape].position.clone
						x = new_point.x + current_shape.faceup[cont].x
						y = new_point.y + current_shape.faceup[cont].y
						z = new_point.z + current_shape.faceup[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								free_2 = false
							else
								cont2+=1
							end
						end

						if (free_2)
							free = true
						else
							cont+=1
						end
					end

					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							x = new_point.x + current_shape.faceup[cont].x
							y = new_point.y + current_shape.faceup[cont].y
							z = new_point.z + current_shape.faceup[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									free_2 = false
								else
									cont2+=1
								end
							end

							if (free_2)
								free = true
							else
								cont+=1
							end
						end

						if (!free)
							cont_shape += 1
						end
					end
				end

			end
			rule.type_transformation = "face"
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "faceleft"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.faceleft.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.faceleft.length))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.faceleft[cont].x
					y = new_point.y + current_shape.faceleft[cont].y
					z = new_point.z + current_shape.faceleft[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							free_2 = false
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end

				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						x = new_point.x + current_shape.faceleft[cont].x
						y = new_point.y + current_shape.faceleft[cont].y
						z = new_point.z + current_shape.faceleft[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								free_2 = false
							else
								cont2+=1
							end
						end

						if (free_2)
							free = true
						else
							cont+=1
						end
					end

					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.faceleft.length)
					cont = random_2
					while ((!free)&&(cont<current_shape.faceleft.length))
						new_point = current_shape.c[cont_shape].position.clone
						x = new_point.x + current_shape.faceleft[cont].x
						y = new_point.y + current_shape.faceleft[cont].y
						z = new_point.z + current_shape.faceleft[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								free_2 = false
							else
								cont2+=1
							end
						end

						if (free_2)
							free = true
						else
							cont+=1
						end
					end

					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							x = new_point.x + current_shape.faceleft[cont].x
							y = new_point.y + current_shape.faceleft[cont].y
							z = new_point.z + current_shape.faceleft[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									free_2 = false
								else
									cont2+=1
								end
							end

							if (free_2)
								free = true
							else
								cont+=1
							end
						end

						if (!free)
							cont_shape += 1
						end
					end
				end

			end
			rule.type_transformation = "face"
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "edge"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.edge.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.edge.length))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.edge[cont].x
					y = new_point.y + current_shape.edge[cont].y
					z = new_point.z + current_shape.edge[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					if current_shape.type.include? "triangle"
						turned = current_shape.c[cont_shape].turned
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if current_shape.c[cont2].turned == turned
									free_2 = false
								else
									cont2+=1
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						x = new_point.x + current_shape.edge[cont].x
						y = new_point.y + current_shape.edge[cont].y
						z = new_point.z + current_shape.edge[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						if current_shape.type.include? "triangle"
							turned = current_shape.c[cont_shape].turned
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if current_shape.c[cont2].turned == turned
										free_2 = false
									else
										cont2+=1
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.edge.length)
					cont = random_2
					while ((!free)&&(cont<current_shape.edge.length))
						new_point = current_shape.c[cont_shape].position.clone
						x = new_point.x + current_shape.edge[cont].x
						y = new_point.y + current_shape.edge[cont].y
						z = new_point.z + current_shape.edge[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						if current_shape.type.include? "triangle"
							turned = current_shape.c[cont_shape].turned
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if current_shape.c[cont2].turned == turned
										free_2 = false
									else
										cont2+=1
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							x = new_point.x + current_shape.edge[cont].x
							y = new_point.y + current_shape.edge[cont].y
							z = new_point.z + current_shape.edge[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							if current_shape.type.include? "triangle"
								turned = current_shape.c[cont_shape].turned
							end
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									if current_shape.type.include? "triangle"
										if current_shape.c[cont2].turned == turned
											free_2 = false
										else
											cont2+=1
										end
									else
										free_2 = false
									end
								else
									cont2+=1
								end
							end
							if (free_2)
								free = true
							else
								cont+=1
							end
						end
						if (!free)
							cont_shape += 1
						end
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "edgeup"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.edgeup.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.edgeup.length))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.edgeup[cont].x
					y = new_point.y + current_shape.edgeup[cont].y
					z = new_point.z + current_shape.edgeup[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							free_2 = false
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						x = new_point.x + current_shape.edgeup[cont].x
						y = new_point.y + current_shape.edgeup[cont].y
						z = new_point.z + current_shape.edgeup[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								free_2 = false
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.edgeup.length)
					cont = random_2
					while ((!free)&&(cont<current_shape.edgeup.length))
						new_point = current_shape.c[cont_shape].position.clone
						x = new_point.x + current_shape.edgeup[cont].x
						y = new_point.y + current_shape.edgeup[cont].y
						z = new_point.z + current_shape.edgeup[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								free_2 = false
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							x = new_point.x + current_shape.edgeup[cont].x
							y = new_point.y + current_shape.edgeup[cont].y
							z = new_point.z + current_shape.edgeup[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									free_2 = false
								else
									cont2+=1
								end
							end
							if (free_2)
								free = true
							else
								cont+=1
							end
						end
						if (!free)
							cont_shape += 1
						end
					end
				end
			end
			rule.type_transformation = "edge"
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "edgeleft"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.edgeleft.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.edgeleft.length))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.edgeleft[cont].x
					y = new_point.y + current_shape.edgeleft[cont].y
					z = new_point.z + current_shape.edgeleft[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							free_2 = false
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						x = new_point.x + current_shape.edgeleft[cont].x
						y = new_point.y + current_shape.edgeleft[cont].y
						z = new_point.z + current_shape.edgeleft[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								free_2 = false
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.edgeleft.length)
					cont = random_2
					while ((!free)&&(cont<current_shape.edgeup.length))
						new_point = current_shape.c[cont_shape].position.clone
						x = new_point.x + current_shape.edgeleft[cont].x
						y = new_point.y + current_shape.edgeleft[cont].y
						z = new_point.z + current_shape.edgeleft[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								free_2 = false
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							x = new_point.x + current_shape.edgeleft[cont].x
							y = new_point.y + current_shape.edgeleft[cont].y
							z = new_point.z + current_shape.edgeleft[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									free_2 = false
								else
									cont2+=1
								end
							end
							if (free_2)
								free = true
							else
								cont+=1
							end
						end
						if (!free)
							cont_shape += 1
						end
					end
				end
			end
			rule.type_transformation = "edge"
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "edge3"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.turnededge2.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.turnededge2.length))
					new_point = current_shape.c[cont_shape].position.clone
					if current_shape.type.include? "triangle"
						turned = !current_shape.c[cont_shape].turned
						if turned == false
							x = new_point.x - current_shape.turnededge2[cont].x
							y = new_point.y - current_shape.turnededge2[cont].y
							z = new_point.z - current_shape.turnededge2[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						else
							x = new_point.x + current_shape.turnededge2[cont].x
							y = new_point.y + current_shape.turnededge2[cont].y
							z = new_point.z + current_shape.turnededge2[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
					else
						x = new_point.x + current_shape.turnededge2[cont].x
						y = new_point.y + current_shape.turnededge2[cont].y
						z = new_point.z + current_shape.turnededge2[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if turned
									if current_shape.c[cont2].turned == true
										free_2 = false
									else
										cont2+=1
									end
								else
									if current_shape.c[cont2].turned == false
										free_2 = false
									else
										cont2+=1
									end
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						if current_shape.type.include? "triangle"
							turned = !current_shape.c[random].turned
							if turned == false
								x = new_point.x - current_shape.turnededge2[cont].x
								y = new_point.y - current_shape.turnededge2[cont].y
								z = new_point.z - current_shape.turnededge2[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							else
								x = new_point.x + current_shape.turnededge2[cont].x
								y = new_point.y + current_shape.turnededge2[cont].y
								z = new_point.z + current_shape.turnededge2[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							end
						else
							x = new_point.x + current_shape.turnededge2[cont].x
							y = new_point.y + current_shape.turnededge2[cont].y
							z = new_point.z + current_shape.turnededge2[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if turned
										if current_shape.c[cont2].turned == true
											free_2 = false
										else
											cont2+=1
										end
									else
										if current_shape.c[cont2].turned == false
											free_2 = false
										else
											cont2+=1
										end
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.turnededge2.length)
					while ((!free)&&(cont<current_shape.turnededge2.length))
						new_point = current_shape.c[cont_shape].position.clone
						if current_shape.type.include? "triangle"
							turned = !current_shape.c[cont_shape].turned
							if turned == false
								x = new_point.x - current_shape.turnededge2[cont].x
								y = new_point.y - current_shape.turnededge2[cont].y
								z = new_point.z - current_shape.turnededge2[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							else
								x = new_point.x + current_shape.turnededge2[cont].x
								y = new_point.y + current_shape.turnededge2[cont].y
								z = new_point.z + current_shape.turnededge2[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							end
						else
							x = new_point.x + current_shape.turnededge2[cont].x
							y = new_point.y + current_shape.turnededge2[cont].y
							z = new_point.z + current_shape.turnededge2[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if turned
										if current_shape.c[cont2].turned == true
											free_2 = false
										else
											cont2+=1
										end
									else
										if current_shape.c[cont2].turned == false
											free_2 = false
										else
											cont2+=1
										end
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							if current_shape.type.include? "triangle"
								turned = !current_shape.c[random].turned
								if turned == false
									x = new_point.x - current_shape.turnededge2[cont].x
									y = new_point.y - current_shape.turnededge2[cont].y
									z = new_point.z - current_shape.turnededge2[cont].z
									new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
								else
									x = new_point.x + current_shape.turnededge2[cont].x
									y = new_point.y + current_shape.turnededge2[cont].y
									z = new_point.z + current_shape.turnededge2[cont].z
									new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
								end
							else
								x = new_point.x + current_shape.turnededge2[cont].x
								y = new_point.y + current_shape.turnededge2[cont].y
								z = new_point.z + current_shape.turnededge2[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							end
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									if current_shape.type.include? "triangle"
										if turned
											if current_shape.c[cont2].turned == true
												free_2 = false
											else
												cont2+=1
											end
										else
											if current_shape.c[cont2].turned == false
												free_2 = false
											else
												cont2+=1
											end
										end
									else
										free_2 = false
									end
								else
									cont2+=1
								end
							end
							if (free_2)
								free = true
							else
								cont+=1
							end
						end
						if (!free)
							cont_shape += 1
						end
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "edge2"
			free = false
			cont_shape = random
			while ((!free)&&(cont_shape<current_shape.c.length))
				random_2 = rand(current_shape.turnededge.length)
				cont = random_2
				while ((!free)&&(cont<current_shape.turnededge.length))
					new_point = current_shape.c[cont_shape].position.clone
					if current_shape.type.include? "triangle"
						turned = !current_shape.c[cont_shape].turned
						if turned == false
							x = new_point.x - current_shape.turnededge[cont].x
							y = new_point.y - current_shape.turnededge[cont].y
							z = new_point.z - current_shape.turnededge[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						else
							x = new_point.x + current_shape.turnededge[cont].x
							y = new_point.y + current_shape.turnededge[cont].y
							z = new_point.z + current_shape.turnededge[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
					else
						x = new_point.x + current_shape.turnededge[cont].x
						y = new_point.y + current_shape.turnededge[cont].y
						z = new_point.z + current_shape.turnededge[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if turned
									if current_shape.c[cont2].turned == true
										free_2 = false
									else
										cont2+=1
									end
								else
									if current_shape.c[cont2].turned == false
										free_2 = false
									else
										cont2+=1
									end
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
				#We haven't found a free place and we must search in the other middle
				if !free
					cont = 0
					free = false
					while ((!free)&&(cont<random_2))
						new_point = current_shape.c[random].position.clone
						if current_shape.type.include? "triangle"
							turned = !current_shape.c[random].turned
							if turned == false
								x = new_point.x - current_shape.turnededge[cont].x
								y = new_point.y - current_shape.turnededge[cont].y
								z = new_point.z - current_shape.turnededge[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							else
								x = new_point.x + current_shape.turnededge[cont].x
								y = new_point.y + current_shape.turnededge[cont].y
								z = new_point.z + current_shape.turnededge[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							end
						else
							x = new_point.x + current_shape.turnededge[cont].x
							y = new_point.y + current_shape.turnededge[cont].y
							z = new_point.z + current_shape.turnededge[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if turned
										if current_shape.c[cont2].turned == true
											free_2 = false
										else
											cont2+=1
										end
									else
										if current_shape.c[cont2].turned == false
											free_2 = false
										else
											cont2+=1
										end
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					if (!free)
						cont_shape += 1
					end
				end
			end
			#We have to search in the other part of the current_shape
			if !free
				cont_shape = 0
				while ((!free)&&(cont_shape<random))
					random_2 = rand(current_shape.turnededge.length)
					while ((!free)&&(cont<current_shape.turnededge.length))
						new_point = current_shape.c[cont_shape].position.clone
						if current_shape.type.include? "triangle"
							turned = !current_shape.c[cont_shape].turned
							if turned == false
								x = new_point.x - current_shape.turnededge[cont].x
								y = new_point.y - current_shape.turnededge[cont].y
								z = new_point.z - current_shape.turnededge[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							else
								x = new_point.x + current_shape.turnededge[cont].x
								y = new_point.y + current_shape.turnededge[cont].y
								z = new_point.z + current_shape.turnededge[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							end
						else
							x = new_point.x + current_shape.turnededge[cont].x
							y = new_point.y + current_shape.turnededge[cont].y
							z = new_point.z + current_shape.turnededge[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
						free_2 = true
						cont2 = 0
						while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
							if (new_point == current_shape.c[cont2].position)
								if current_shape.type.include? "triangle"
									if turned
										if current_shape.c[cont2].turned == true
											free_2 = false
										else
											cont2+=1
										end
									else
										if current_shape.c[cont2].turned == false
											free_2 = false
										else
											cont2+=1
										end
									end
								else
									free_2 = false
								end
							else
								cont2+=1
							end
						end
						if (free_2)
							free = true
						else
							cont+=1
						end
					end
					#We haven't found a free place and we must search in the other middle
					if !free
						cont = 0
						free = false
						while ((!free)&&(cont<random_2))
							new_point = current_shape.c[random].position.clone
							if current_shape.type.include? "triangle"
								turned = !current_shape.c[random].turned
								if turned == false
									x = new_point.x - current_shape.turnededge[cont].x
									y = new_point.y - current_shape.turnededge[cont].y
									z = new_point.z - current_shape.turnededge[cont].z
									new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
								else
									x = new_point.x + current_shape.turnededge[cont].x
									y = new_point.y + current_shape.turnededge[cont].y
									z = new_point.z + current_shape.turnededge[cont].z
									new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
								end
							else
								x = new_point.x + current_shape.turnededge[cont].x
								y = new_point.y + current_shape.turnededge[cont].y
								z = new_point.z + current_shape.turnededge[cont].z
								new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
							end
							free_2 = true
							cont2 = 0
							while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
								if (new_point == current_shape.c[cont2].position)
									if current_shape.type.include? "triangle"
										if turned
											if current_shape.c[cont2].turned == true
												free_2 = false
											else
												cont2+=1
											end
										else
											if current_shape.c[cont2].turned == false
												free_2 = false
											else
												cont2+=1
											end
										end
									else
										free_2 = false
									end
								else
									cont2+=1
								end
							end
							if (free_2)
								free = true
							else
								cont+=1
							end
						end
						if (!free)
							cont_shape += 1
						end
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		end
	end
	
	#rule_id:: internal id of the rule to apply to the current shape
	#cont_shape:: array index of the shape to apply the rule
	#current_shape:: the current design
	#backtraking:: true if we're using a backtraking process
	#
	#Applies the specified rule to the specified shape.
	def apply_rule_shape(rule_id, cont_shape, current_shape, backtraking)
		rule = @grammar.search_rule_by_id(rule_id)
		if current_shape.type.include? "cube"
			if @up && @left
				rule = @grammar.search_rule_by_id(rule_id)
			elsif @up
				if rule.type_transformation == "face"
					rule.type_transformation = "faceup"
				elsif rule.type_transformation == "edge"
					rule.type_transformation = "edgeup"
				end
			elsif @left
				if rule.type_transformation == "face"
					rule.type_transformation = "faceleft"
				elsif rule.type_transformation == "edge"
					rule.type_transformation = "edgeleft"
				end
			end
		end
		if rule.type_transformation == "point"
			random_2 = rand(current_shape.point.length)
			cont = random_2
			free = false
			while ((!free)&&(cont<current_shape.point.length))
				new_point = current_shape.c[cont_shape].position.clone
				x = new_point.x + current_shape.point[cont].x
				y = new_point.y + current_shape.point[cont].y
				z = new_point.z + current_shape.point[cont].z
				new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
				if current_shape.type.include? "triangle"
					turned = current_shape.c[cont_shape].turned
				end
				free_2 = true
				cont2 = 0
				while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
					if (new_point == current_shape.c[cont2].position)
						if current_shape.type.include? "triangle"
							if current_shape.c[cont2].turned == turned
								free_2 = false
							else
								cont2+=1
							end
						else
							free_2 = false
						end
					else
						cont2+=1
					end
				end
				if (free_2)
					free = true
				else
					cont+=1
				end
			end
			#We haven't found a free place and we must search in the other middle
			if !free
				cont = 0
				free = false
				while ((!free)&&(cont<random_2))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.point[cont].x
					y = new_point.y + current_shape.point[cont].y
					z = new_point.z + current_shape.point[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					if current_shape.type.include? "triangle"
						turned = current_shape.c[cont_shape].turned
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if current_shape.c[cont2].turned == turned
									free_2 = false
								else
									cont2+=1
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "edge"
			random_2 = rand(current_shape.edge.length)
			cont = random_2
			free = false
			while ((!free)&&(cont<current_shape.edge.length))
				new_point = current_shape.c[cont_shape].position.clone
				x = new_point.x + current_shape.edge[cont].x
				y = new_point.y + current_shape.edge[cont].y
				z = new_point.z + current_shape.edge[cont].z
				new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
				if current_shape.type.include? "triangle"
					turned = current_shape.c[cont_shape].turned
				end
				free_2 = true
				cont2 = 0
				while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
					if (new_point == current_shape.c[cont2].position)
						if current_shape.type.include? "triangle"
							if current_shape.c[cont2].turned == turned
								free_2 = false
							else
								cont2+=1
							end
						else
							free_2 = false
						end
					else
						cont2+=1
					end
				end
				if (free_2)
					free = true
				else
					cont+=1
				end
			end
			#We haven't found a free place and we must search in the other middle
			if !free
				cont = 0
				free = false
				while ((!free)&&(cont<random_2))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.edge[cont].x
					y = new_point.y + current_shape.edge[cont].y
					z = new_point.z + current_shape.edge[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					if current_shape.type.include? "triangle"
						turned = current_shape.c[cont_shape].turned
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if current_shape.c[cont2].turned == turned
									free_2 = false
								else
									cont2+=1
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "edgeup"
			random_2 = rand(current_shape.edgeup.length)
			cont = random_2
			free = false
			while ((!free)&&(cont<current_shape.edgeup.length))
				new_point = current_shape.c[cont_shape].position.clone
				x = new_point.x + current_shape.edgeup[cont].x
				y = new_point.y + current_shape.edgeup[cont].y
				z = new_point.z + current_shape.edgeup[cont].z
				new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
				free_2 = true
				cont2 = 0
				while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
					if (new_point == current_shape.c[cont2].position)
						free_2 = false
					else
						cont2+=1
					end
				end
				if (free_2)
					free = true
				else
					cont+=1
				end
			end
			#We haven't found a free place and we must search in the other middle
			if !free
				cont = 0
				free = false
				while ((!free)&&(cont<random_2))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.edgeup[cont].x
					y = new_point.y + current_shape.edgeup[cont].y
					z = new_point.z + current_shape.edgeup[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							free_2 = false
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
			end
			rule.type_transformation = "edge"
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "edgeleft"
			random_2 = rand(current_shape.edgeleft.length)
			cont = random_2
			free = false
			while ((!free)&&(cont<current_shape.edgeleft.length))
				new_point = current_shape.c[cont_shape].position.clone
				x = new_point.x + current_shape.edgeleft[cont].x
				y = new_point.y + current_shape.edgeleft[cont].y
				z = new_point.z + current_shape.edgeleft[cont].z
				new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
				free_2 = true
				cont2 = 0
				while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
					if (new_point == current_shape.c[cont2].position)
						free_2 = false
					else
						cont2+=1
					end
				end
				if (free_2)
					free = true
				else
					cont+=1
				end
			end
			#We haven't found a free place and we must search in the other middle
			if !free
				cont = 0
				free = false
				while ((!free)&&(cont<random_2))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.edgeleft[cont].x
					y = new_point.y + current_shape.edgeleft[cont].y
					z = new_point.z + current_shape.edgeleft[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							free_2 = false
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
			end
			rule.type_transformation = "edge"
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "face"
			random_2 = rand(current_shape.face.length)
			cont = random_2
			free = false
			while ((!free)&&(cont<current_shape.face.length))
				new_point = current_shape.c[cont_shape].position.clone
				x = new_point.x + current_shape.face[cont].x
				y = new_point.y + current_shape.face[cont].y
				z = new_point.z + current_shape.face[cont].z
				new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
				if current_shape.type.include? "triangle"
					turned = current_shape.c[cont_shape].turned
				end
				free_2 = true
				cont2 = 0
				while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
					if (new_point == current_shape.c[cont2].position)
						if current_shape.type.include? "triangle"
							if current_shape.c[cont2].turned ==turned
								free_2 = false
							else
								cont2+=1
							end
						else
							free_2 = false
						end
					else
						cont2+=1
					end
				end
				if (free_2)
					free = true
				else
					cont+=1
				end
			end

			#We haven't found a free place and we must search in the other middle
			if !free
				cont = 0
				free = false
				while ((!free)&&(cont<random_2))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.face[cont].x
					y = new_point.y + current_shape.face[cont].y
					z = new_point.z + current_shape.face[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					if current_shape.type.include? "triangle"
						turned = current_shape.c[cont_shape].turned
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if current_shape.c[cont2].turned ==turned
									free_2 = false
								else
									cont2+=1
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "faceup"
			random_2 = rand(current_shape.faceup.length)
			cont = random_2
			free = false
			while ((!free)&&(cont<current_shape.faceup.length))
				new_point = current_shape.c[cont_shape].position.clone
				x = new_point.x + current_shape.faceup[cont].x
				y = new_point.y + current_shape.faceup[cont].y
				z = new_point.z + current_shape.faceup[cont].z
				new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
				free_2 = true
				cont2 = 0
				while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
					if (new_point == current_shape.c[cont2].position)
						free_2 = false
					else
						cont2+=1
					end
				end
				if (free_2)
					free = true
				else
					cont+=1
				end
			end

			#We haven't found a free place and we must search in the other middle
			if !free
				cont = 0
				free = false
				while ((!free)&&(cont<random_2))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.faceup[cont].x
					y = new_point.y + current_shape.faceup[cont].y
					z = new_point.z + current_shape.faceup[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							free_2 = false
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
			end
			rule.type_transformation = "face"
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "faceleft"
			random_2 = rand(current_shape.faceleft.length)
			cont = random_2
			free = false
			while ((!free)&&(cont<current_shape.faceleft.length))
				new_point = current_shape.c[cont_shape].position.clone
				x = new_point.x + current_shape.faceleft[cont].x
				y = new_point.y + current_shape.faceleft[cont].y
				z = new_point.z + current_shape.faceleft[cont].z
				new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
				free_2 = true
				cont2 = 0
				while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
					if (new_point == current_shape.c[cont2].position)
						free_2 = false
					else
						cont2+=1
					end
				end
				if (free_2)
					free = true
				else
					cont+=1
				end
			end

			#We haven't found a free place and we must search in the other middle
			if !free
				cont = 0
				free = false
				while ((!free)&&(cont<random_2))
					new_point = current_shape.c[cont_shape].position.clone
					x = new_point.x + current_shape.faceleft[cont].x
					y = new_point.y + current_shape.faceleft[cont].y
					z = new_point.z + current_shape.faceleft[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							free_2 = false
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
			end
			rule.type_transformation = "face"
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "face2"
			random_2 = rand(current_shape.turnedface.length)
			cont = random_2
			free = false
			while ((!free)&&(cont<current_shape.turnedface.length))
				new_point = current_shape.c[cont_shape].position.clone
				if current_shape.type.include? "triangle"
					turned = !current_shape.c[cont_shape].turned
					if turned == false
						x = new_point.x - current_shape.turnedface[cont].x
						y = new_point.y - current_shape.turnedface[cont].y
						z = new_point.z - current_shape.turnedface[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					else
						x = new_point.x + current_shape.turnedface[cont].x
						y = new_point.y + current_shape.turnedface[cont].y
						z = new_point.z + current_shape.turnedface[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					end
				else
					x = new_point.x + current_shape.turnedface[cont].x
					y = new_point.y + current_shape.turnedface[cont].y
					z = new_point.z + current_shape.turnedface[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
				end
				free_2 = true
				cont2 = 0
				while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
					if (new_point == current_shape.c[cont2].position)
						if current_shape.type.include? "triangle"
							if !turned
								if current_shape.c[cont2].turned == false
									free_2 = false
								else
									cont2+=1
								end
							else
								if current_shape.c[cont2].turned
									free_2 = false
								else
									cont2+=1
								end
							end
						else
							free_2 = false
						end
					else
						cont2+=1
					end
				end
				if (free_2)
					free = true
				else
					cont+=1
				end
			end
			#We haven't found a free place and we must search in the other middle
			if !free
				cont = 0
				free = false
				while ((!free)&&(cont<random_2))
					new_point = current_shape.c[cont_shape].position.clone
					if current_shape.type.include? "triangle"
						turned = !current_shape.c[cont_shape].turned
						if turned == false
							x = new_point.x - current_shape.turnedface[cont].x
							y = new_point.y - current_shape.turnedface[cont].y
							z = new_point.z - current_shape.turnedface[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						else
							x = new_point.x + current_shape.turnedface[cont].x
							y = new_point.y + current_shape.turnedface[cont].y
							z = new_point.z + current_shape.turnedface[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
					else
						x = new_point.x + current_shape.turnedface[cont].x
						y = new_point.y + current_shape.turnedface[cont].y
						z = new_point.z + current_shape.turnedface[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if !turned
									if current_shape.c[cont2].turned == false
										free_2 = false
									else
										cont2+=1
									end
								else
									if current_shape.c[cont2].turned
										free_2 = false
									else
										cont2+=1
									end
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "point2"
			random_2 = rand(current_shape.turnedpoint.length)
			cont = random_2
			free = false
			while ((!free)&&(cont<current_shape.turnedpoint.length))
				new_point = current_shape.c[cont_shape].position.clone
				if current_shape.type.include? "triangle"
					turned = !current_shape.c[cont_shape].turned
					if turned == false
						x = new_point.x - current_shape.turnedpoint[cont].x
						y = new_point.y - current_shape.turnedpoint[cont].y
						z = new_point.z - current_shape.turnedpoint[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					else
						x = new_point.x + current_shape.turnedpoint[cont].x
						y = new_point.y + current_shape.turnedpoint[cont].y
						z = new_point.z + current_shape.turnedpoint[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					end
				else
					x = new_point.x + current_shape.turnedpoint[cont].x
					y = new_point.y + current_shape.turnedpoint[cont].y
					z = new_point.z + current_shape.turnedpoint[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
				end
				free_2 = true
				cont2 = 0
				while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
					if (new_point == current_shape.c[cont2].position)
						if current_shape.type.include? "triangle"
							if !turned
								if current_shape.c[cont2].turned == false
									free_2 = false
								else
									cont2+=1
								end
							else
								if current_shape.c[cont2].turned
									free_2 = false
								else
									cont2+=1
								end
							end
						else
							free_2 = false
						end
					else
						cont2+=1
					end
				end
				if (free_2)
					free = true
				else
					cont+=1
				end
			end
			#We haven't found a free place and we must search in the other middle
			if !free
				cont = 0
				free = false
				while ((!free)&&(cont<random_2))
					new_point = current_shape.c[cont_shape].position.clone
					if current_shape.type.include? "triangle"
						turned = !current_shape.c[cont_shape].turned
						if turned == false
							x = new_point.x - current_shape.turnedpoint[cont].x
							y = new_point.y - current_shape.turnedpoint[cont].y
							z = new_point.z - current_shape.turnedpoint[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						else
							x = new_point.x + current_shape.turnedpoint[cont].x
							y = new_point.y + current_shape.turnedpoint[cont].y
							z = new_point.z + current_shape.turnedpoint[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
					else
						x = new_point.x + current_shape.turnedpoint[cont].x
						y = new_point.y + current_shape.turnedpoint[cont].y
						z = new_point.z + current_shape.turnedpoint[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if !turned
									if current_shape.c[cont2].turned == false
										free_2 = false
									else
										cont2+=1
									end
								else
									if current_shape.c[cont2].turned
										free_2 = false
									else
										cont2+=1
									end
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "edge3"
			random_2 = rand(current_shape.turnededge2.length)
			cont = random_2
			free = false
			while ((!free)&&(cont<current_shape.turnededge2.length))
				new_point = current_shape.c[cont_shape].position.clone
				if current_shape.type.include? "triangle"
					turned = !current_shape.c[cont_shape].turned
					if turned == false
						x = new_point.x - current_shape.turnededge2[cont].x
						y = new_point.y - current_shape.turnededge2[cont].y
						z = new_point.z - current_shape.turnededge2[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					else
						x = new_point.x + current_shape.turnededge2[cont].x
						y = new_point.y + current_shape.turnededge2[cont].y
						z = new_point.z + current_shape.turnededge2[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					end
				else
					x = new_point.x + current_shape.turnededge2[cont].x
					y = new_point.y + current_shape.turnededge2[cont].y
					z = new_point.z + current_shape.turnededge2[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
				end
				free_2 = true
				cont2 = 0
				while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
					if (new_point == current_shape.c[cont2].position)
						if current_shape.type.include? "triangle"
							if !turned
								if current_shape.c[cont2].turned == false
									free_2 = false
								else
									cont2+=1
								end
							else
								if current_shape.c[cont2].turned
									free_2 = false
								else
									cont2+=1
								end
							end
						else
							free_2 = false
						end
					else
						cont2+=1
					end
				end
				if (free_2)
					free = true
				else
					cont+=1
				end
			end
			#We haven't found a free place and we must search in the other middle
			if !free
				cont = 0
				free = false
				while ((!free)&&(cont<random_2))
					new_point = current_shape.c[cont_shape].position.clone
					if current_shape.type.include? "triangle"
						turned = !current_shape.c[cont_shape].turned
						if turned == false
							x = new_point.x - current_shape.turnededge2[cont].x
							y = new_point.y - current_shape.turnededge2[cont].y
							z = new_point.z - current_shape.turnededge2[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						else
							x = new_point.x + current_shape.turnededge2[cont].x
							y = new_point.y + current_shape.turnededge2[cont].y
							z = new_point.z + current_shape.turnededge2[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
					else
						x = new_point.x + current_shape.turnededge2[cont].x
						y = new_point.y + current_shape.turnededge2[cont].y
						z = new_point.z + current_shape.turnededge2[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if !turned
									if current_shape.c[cont2].turned == false
										free_2 = false
									else
										cont2+=1
									end
								else
									if current_shape.c[cont2].turned
										free_2 = false
									else
										cont2+=1
									end
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		elsif rule.type_transformation == "edge2"
			random_2 = rand(current_shape.turnededge.length)
			cont = random_2
			free = false
			while ((!free)&&(cont<current_shape.turnededge.length))
				new_point = current_shape.c[cont_shape].position.clone
				if current_shape.type.include? "triangle"
					turned = !current_shape.c[cont_shape].turned
					if turned == false
						x = new_point.x - current_shape.turnededge[cont].x
						y = new_point.y - current_shape.turnededge[cont].y
						z = new_point.z - current_shape.turnededge[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					else
						x = new_point.x + current_shape.turnededge[cont].x
						y = new_point.y + current_shape.turnededge[cont].y
						z = new_point.z + current_shape.turnededge[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					end
				else
					x = new_point.x + current_shape.turnededge[cont].x
					y = new_point.y + current_shape.turnededge[cont].y
					z = new_point.z + current_shape.turnededge[cont].z
					new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
				end
				free_2 = true
				cont2 = 0
				while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
					if (new_point == current_shape.c[cont2].position)
						if current_shape.type.include? "triangle"
							if !turned
								if current_shape.c[cont2].turned == false
									free_2 = false
								else
									cont2+=1
								end
							else
								if current_shape.c[cont2].turned
									free_2 = false
								else
									cont2+=1
								end
							end
						else
							free_2 = false
						end
					else
						cont2+=1
					end
				end
				if (free_2)
					free = true
				else
					cont+=1
				end
			end
			#We haven't found a free place and we must search in the other middle
			if !free
				cont = 0
				free = false
				while ((!free)&&(cont<random_2))
					new_point = current_shape.c[cont_shape].position.clone
					if current_shape.type.include? "triangle"
						turned = !current_shape.c[cont_shape].turned
						if turned == false
							x = new_point.x - current_shape.turnededge[cont].x
							y = new_point.y - current_shape.turnededge[cont].y
							z = new_point.z - current_shape.turnededge[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						else
							x = new_point.x + current_shape.turnededge[cont].x
							y = new_point.y + current_shape.turnededge[cont].y
							z = new_point.z + current_shape.turnededge[cont].z
							new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
						end
					else
						x = new_point.x + current_shape.turnededge[cont].x
						y = new_point.y + current_shape.turnededge[cont].y
						z = new_point.z + current_shape.turnededge[cont].z
						new_point = OrderedPoint.new(Geom::Point3d.new(x, y, z))
					end
					free_2 = true
					cont2 = 0
					while ((free_2)&&(cont2 < current_shape.c.length)&&(new_point >= current_shape.c[cont2].position))
						if (new_point == current_shape.c[cont2].position)
							if current_shape.type.include? "triangle"
								if !turned
									if current_shape.c[cont2].turned == false
										free_2 = false
									else
										cont2+=1
									end
								else
									if current_shape.c[cont2].turned
										free_2 = false
									else
										cont2+=1
									end
								end
							else
								free_2 = false
							end
						else
							cont2+=1
						end
					end
					if (free_2)
						free = true
					else
						cont+=1
					end
				end
			end
			if free
				if backtraking
					if current_shape.type.include? "cube"
						return (PositionCube.new(current_shape.c[0].cube.duplicate, new_point))
					elsif current_shape.type.include? "triangle"
						return (PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
					elsif current_shape.type.include? "hexagonal"
						return (PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
					end
				else
					@execution_history.push [rule_id, @current_shape.duplicate]
					if @current_shape.type.include? "cube"
						@current_shape.c.push(PositionCube.new(@current_shape.c[0].cube.duplicate, new_point))
						@current_shape.repaint
					elsif @current_shape.type.include? "triangle"
						@current_shape.c.push(PositionTriangularPrism.new(@current_shape.c[0].prism.duplicate, new_point, turned))
						@current_shape.repaint
					elsif @current_shape.type.include? "hexagonal"
						@current_shape.c.push(PositionHexagonalPrism.new(@current_shape.c[0].prism.duplicate, new_point))
						@current_shape.repaint
					end
				end
			end
		end
	end
	
	
	#rule_id:: internal id of the rule to apply to the current shape
	#
	#Applies rules in a backtracking process
	def backtracking(rule_id, backtracking=false)
		if @num_shapes && (@max_num_shapes < @current_shape.c.length + 1)
			solution = false
		else
			@current_shape.c.sort! { |shape1, shape2|
				shape1.position <=> shape2.position
			}
			current_shape = @current_shape.duplicate
			solution = false
			while ((!solution)&&(!current_shape.c.empty?))
				random = rand(current_shape.c.length-1)
				if @current_shape.type.include? "triangle"
					position = @current_shape.position(current_shape.c[random-1].position, current_shape.c[random-1].turned)
				else
					position = @current_shape.position(current_shape.c[random-1].position)
				end
				solution = self.backtracking_shape(rule_id, position, backtracking)
				if !solution
					current_shape.c.delete_at(random-1)
				end
			end
		end
		return solution
	end
	
	#rule_id:: internal id of the rule to apply to the current shape
	#current_shape:: the specified current design
	#
	#Applies rules in a backtracking process to a specified current shape
	def backtracking_sel(rule_id, current_shape, backtracking=false)
		if @num_shapes && (@max_num_shapes < @current_shape.c.length + 1)
			solution = false
		else
			current_shape.c.sort! { |shape1, shape2|
				shape1.position <=> shape2.position
			}
			current = current_shape.duplicate
			solution = false
			while ((!solution)&&(!current.c.empty?))
				random = rand(current.c.length-1)
				if current_shape.type.include? "triangle"
					position = current_shape.position(current.c[random-1].position, current.c[random-1].turned)
				else
					position = current_shape.position(current.c[random-1].position)
				end
				solution = self.backtracking_shape_sel(rule_id, position, current_shape, backtracking)
				if !solution
					current.c.delete_at(random-1)
				end
			end
		end
		return solution
	end
	
	#rule_id:: internal id of the rule to apply to the current shape
	#cont_shape:: the index of a specified shape
	#current_shape:: the specified current design
	#
	#Applies rules in a backtracking process to a specified shape from a specified current shape
	def backtracking_shape_sel(rule_id, cont_shape, current_shape, backtracking=false)
		if @num_shapes && (@max_num_shapes < @current_shape.c.length + 1)
			solution = false
		else
			current = current_shape.duplicate
			current.c.sort! { |shape1, shape2|
				shape1.position <=> shape2.position
			}
			solution = false
			rule = @grammar.search_rule_by_id(rule_id)
			if current.type.include? "cube"
				if @up && @left
					rule = @grammar.search_rule_by_id(rule_id)
				elsif @up
					if rule.type_transformation == "face"
						rule.type_transformation = "faceup"
					elsif rule.type_transformation == "edge"
						rule.type_transformation = "edgeup"
					end
				elsif @left
					if rule.type_transformation == "face"
						rule.type_transformation = "faceleft"
					elsif rule.type_transformation == "edge"
						rule.type_transformation = "edgeleft"
					end
				end
			end
			if rule.type_transformation == "face"
				salir = false
				while ((!current.face.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension &&candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.face.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate)&& !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.face.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.face.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.face.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)&&candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.face.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.face.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.face.delete(point)
						end
					end
				end
			elsif rule.type_transformation == "faceup"
				salir = false
				while ((!current.faceup.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate) && candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceup.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceup.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceup.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceup.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceup.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceup.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceup.delete(point)
						end
					end
				end
				rule.type_transformation = "face"
			elsif rule.type_transformation == "faceleft"
				salir = false
				while ((!current.faceleft.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceleft.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceleft.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceleft.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceleft.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceleft.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceleft.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.faceleft.delete(point)
						end
					end
				end
				rule.type_transformation = "face"
			elsif rule.type_transformation == "face2"
				salir = false
				while ((!current.turnedface.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedface.delete(point)
							end
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedface.delete(point)
							end
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedface.delete(point)
							end
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedface.delete(point)
							end
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedface.delete(point)
							end
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedface.delete(point)
							end
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedface.delete(point)
							end
						end
					end
				end
			elsif rule.type_transformation == "edge2"
				salir = false
				while ((!current.turnededge.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge.delete(point)
							end
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge.delete(point)
							end
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge.delete(point)
							end
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge.delete(point)
							end
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge.delete(point)
							end
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge.delete(point)
							end
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge.delete(point)
							end
						end
					end
				end
			elsif rule.type_transformation == "edge3"
				salir = false
				while ((!current.turnededge2.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge2.delete(point)
							end
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge2.delete(point)
							end
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge2.delete(point)
							end
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge2.delete(point)
							end
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge2.delete(point)
							end
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge2.delete(point)
							end
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnededge2.delete(point)
							end
						end
					end
				end
			elsif rule.type_transformation == "edge"
				salir = false
				while ((!current.edge.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edge.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edge.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edge.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edge.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edge.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edge.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edge.delete(point)
						end
					end
				end
			elsif rule.type_transformation == "edgeup"
				salir = false
				while ((!current.edgeup.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeup.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeup.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeup.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeup.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeup.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeup.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeup.delete(point)
						end
					end
				end
				rule.type_transformation = "edge"
			elsif rule.type_transformation == "edgeleft"
				salir = false
				while ((!current.edgeleft.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeleft.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeleft.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeleft.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeleft.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeleft.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeleft.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.edgeleft.delete(point)
						end
					end
				end
				rule.type_transformation = "edge"
			elsif rule.type_transformation == "point2"
				salir = false
				while ((!current.turnedpoint.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedpoint.delete(point)
							end
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedpoint.delete(point)
							end
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedpoint.delete(point)
							end
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedpoint.delete(point)
							end
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedpoint.delete(point)
							end
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedpoint.delete(point)
							end
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							if current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current.c[cont_shape].position.x, -candidate.position.y+current.c[cont_shape].position.y, -candidate.position.z+current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
									current.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
								current.turnedpoint.delete(point)
							end
						end
					end
				end
			elsif rule.type_transformation == "point"
				salir = false
				while ((!current.point.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.point.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.point.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.point.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.point.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.point.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.point.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@current_shape.esta(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current.c[cont_shape].position.x, candidate.position.y-current.c[cont_shape].position.y, candidate.position.z-current.c[cont_shape].position.z))
							current.point.delete(point)
						end
					end
				end
			end
			if solution
				@execution_history.push [rule_id, @current_shape.duplicate]
				if backtracking==false
					@num_steps.push(1)
				end
				current_shape.c.push(candidate)
				@current_shape.c.push(candidate)
				@current_shape.repaint
			end
		end
		return solution
	end
	
	#rule_id:: internal id of the rule to apply to the current shape
	#cont_shape:: the index of a specific shape
	#
	#Applies rules in a backtracking process to a specified shape
	def backtracking_shape(rule_id, cont_shape, backtracking=false)
		if @num_shapes && (@max_num_shapes < @current_shape.c.length + 1)
			solution = false
		else
			current_shape = @current_shape.duplicate
			current_shape.c.sort! { |shape1, shape2|
				shape1.position <=> shape2.position
			}
			solution = false
			rule = @grammar.search_rule_by_id(rule_id)
			if current_shape.type.include? "cube"
				if @up && @left
					rule = @grammar.search_rule_by_id(rule_id)
				elsif @up
					if rule.type_transformation == "face"
						rule.type_transformation = "faceup"
					elsif rule.type_transformation == "edge"
						rule.type_transformation = "edgeup"
					end
				elsif @left
					if rule.type_transformation == "face"
						rule.type_transformation = "faceleft"
					elsif rule.type_transformation == "edge"
						rule.type_transformation = "edgeleft"
					end
				end
			end
			if rule.type_transformation == "face"
				salir = false
				while ((!current_shape.face.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension &&candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.face.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.face.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.face.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.face.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)&&candidate.dimension
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.face.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.face.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.face.delete(point)
						end
					end
				end
			elsif rule.type_transformation == "faceup"
				salir = false
				while ((!current_shape.faceup.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate) && candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceup.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceup.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceup.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceup.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceup.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceup.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceup.delete(point)
						end
					end
				end
				rule.type_transformation = "face"
			elsif rule.type_transformation == "faceleft"
				salir = false
				while ((!current_shape.faceleft.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceleft.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceleft.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceleft.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceleft.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceleft.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceleft.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.faceleft.delete(point)
						end
					end
				end
				rule.type_transformation = "face"
			elsif rule.type_transformation == "face2"
				salir = false
				while ((!current_shape.turnedface.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedface.delete(point)
							end
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedface.delete(point)
							end
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedface.delete(point)
							end
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedface.delete(point)
							end
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedface.delete(point)
							end
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedface.delete(point)
							end
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedface.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedface.delete(point)
							end
						end
					end
				end
			elsif rule.type_transformation == "edge2"
				salir = false
				while ((!current_shape.turnededge.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge.delete(point)
							end
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge.delete(point)
							end
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge.delete(point)
							end
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge.delete(point)
							end
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge.delete(point)
							end
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge.delete(point)
							end
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge.delete(point)
							end
						end
					end
				end
			elsif rule.type_transformation == "edge3"
				salir = false
				while ((!current_shape.turnededge2.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge2.delete(point)
							end
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge2.delete(point)
							end
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge2.delete(point)
							end
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge2.delete(point)
							end
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge2.delete(point)
							end
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge2.delete(point)
							end
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnededge2.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnededge2.delete(point)
							end
						end
					end
				end
			elsif rule.type_transformation == "edge"
				salir = false
				while ((!current_shape.edge.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edge.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edge.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edge.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edge.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edge.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edge.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edge.delete(point)
						end
					end
				end
			elsif rule.type_transformation == "edgeup"
				salir = false
				while ((!current_shape.edgeup.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeup.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeup.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeup.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeup.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeup.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeup.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeup.delete(point)
						end
					end
				end
				rule.type_transformation = "edge"
			elsif rule.type_transformation == "edgeleft"
				salir = false
				while ((!current_shape.edgeleft.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeleft.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeleft.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeleft.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeleft.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeleft.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeleft.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.edgeleft.delete(point)
						end
					end
				end
				rule.type_transformation = "edge"
			elsif rule.type_transformation == "point2"
				salir = false
				while ((!current_shape.turnedpoint.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedpoint.delete(point)
							end
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedpoint.delete(point)
							end
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedpoint.delete(point)
							end
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedpoint.delete(point)
							end
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate) && candidate.dimension
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedpoint.delete(point)
							end
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedpoint.delete(point)
							end
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension
							solution = true
						else
							if @current_shape.type.include? "triangle"
								if !candidate.turned
									point = OrderedPoint.new(Geom::Point3d.new(-candidate.position.x+current_shape.c[cont_shape].position.x, -candidate.position.y+current_shape.c[cont_shape].position.y, -candidate.position.z+current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								else
									point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
									current_shape.turnedpoint.delete(point)
								end
							else
								point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
								current_shape.turnedpoint.delete(point)
							end
						end
					end
				end
			elsif rule.type_transformation == "point"
				salir = false
				while ((!current_shape.point.empty?)&&(!solution)&&(!salir))
					if @area && @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.point.delete(point)
						end
					elsif @area && @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.point.delete(point)
						end
					elsif @area && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.point.delete(point)
						end
					elsif @area
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.area
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.point.delete(point)
						end
					elsif @hill.painted && @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension && !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.point.delete(point)
						end
					elsif @hill.painted
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif !@hill.intersect(candidate)
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.point.delete(point)
						end
					elsif @volumen
						candidate = self.apply_rule_shape(rule_id, cont_shape, current_shape, true)
						if candidate.nil?
							salir = true
						elsif candidate.dimension
							solution = true
						else
							point = OrderedPoint.new(Geom::Point3d.new(candidate.position.x-current_shape.c[cont_shape].position.x, candidate.position.y-current_shape.c[cont_shape].position.y, candidate.position.z-current_shape.c[cont_shape].position.z))
							current_shape.point.delete(point)
						end
					end
				end
			end
			if solution
				@execution_history.push [rule_id, @current_shape.duplicate]
				if backtracking==false
					@num_steps.push(1)
				end
				@current_shape.c.push(candidate)
				@current_shape.repaint
			end
		end
		return solution
	end
	
	#candidate:: a shape
	#
	#Checks if the candidate intersect with the countour line map
	def teste_candidate_intersection (candidate)
		if @hill.painted 
			if !@hill.intersect(candidate) && !@current_shape.esta(candidate)#candidate.area && !@current_shape.esta(candidate.position)
				@current_shape.c.push(candidate)
				@current_shape.repaint
			else
				candidate.position.z = candidate.position.z + Constants::HEIGHT_S
				if !@hill.intersect(candidate)
					@current_shape.c.push(candidate)
					@current_shape.repaint
				else
					UI.messagebox("La semilla colisiona con otras figuras")
				end
			end
		elsif !@current_shape.esta(candidate)#candidate.area && !@current_shape.esta(candidate.position)
			@current_shape.c.push(candidate)
			@current_shape.repaint
		else
			UI.messagebox("La semilla colisiona con otras figuras")
		end
		#this method only is used when we add a seed: we must add the seed to the historial
		cont = 0
		while (cont < @execution_history.length)
			@execution_history[cont][1].c.push(candidate)
			cont += 1
		end
	end
	
	#candidate:: a shape
	#
	#Moves the axiom to the candidate.position
	def move_axiom (candidate)
		if @current_shape.type.include? "triangle"
			position = @current_shape.position(@axiom.position, @axiom.turned)
		else
			position = @current_shape.position(@axiom.position)
		end
		if position >= @current_shape.c.length
			position = 0
		end
		if @hill.painted
			if !@hill.intersect(candidate)&& !@current_shape.esta(candidate)
				@current_shape.c[position].erase
				@current_shape.c.delete_at(position)
				@current_shape.c.push(candidate)
				@axiom = candidate
				@current_shape.repaint
			else
				candidate.position.z = candidate.position.z + Constants::HEIGHT_S
				if !@hill.intersect(candidate) && !@current_shape.esta(candidate)
					@current_shape.c[position].erase
					@current_shape.c.delete_at(position)
					@current_shape.c.push(candidate)
					@axiom = candidate
					@current_shape.repaint
				else
					UI.messagebox("La semilla no se puede mover porque colisiona con las curvas de nivel")
				end
			end
		elsif !@current_shape.esta(candidate)
			@current_shape.c[position].erase
			@current_shape.c.delete_at(position)
			@current_shape.c.push(candidate)
			@axiom = candidate
			@current_shape.repaint
		else
			candidate.position.z = candidate.position.z + Constants::HEIGHT_S
			if !@current_shape.esta(candidate)
				@current_shape.c[position].erase
				@current_shape.c.delete_at(position)
				@current_shape.c.push(candidate)
				@axiom = candidate
				@current_shape.repaint
			else
				UI.messagebox("La semilla no se puede mover porque colisiona con otras figuras")
			end
		end
	end
	
	#candidate:: a shape
	#position:: the index of a specific shape
	#
	#Moves the seed to a candidate.position
	def move_seed (candidate, position)
		@current_shape.c.sort! { |shape1, shape2|
			shape1.position <=> shape2.position
		}
		if @hill.painted
			if !@hill.intersect(candidate)&& !@current_shape.esta(candidate)
				@current_shape.c[position].erase
				@current_shape.c.delete_at(position)
				@current_shape.c.push(candidate)
				@current_shape.repaint
			else
				candidate.position.z = candidate.position.z + Constants::HEIGHT_S
				if !@hill.intersect(candidate)&& !@current_shape.esta(candidate)
					@current_shape.c[position].erase
					@current_shape.c.delete_at(position)
					@current_shape.c.push(candidate)
					@current_shape.repaint
				else
					UI.messagebox("La semilla no se puede mover porque colisiona con las curvas de nivel")
				end
			end
		elsif !@current_shape.esta(candidate)
			@current_shape.c[position].erase
			@current_shape.c.delete_at(position)
			@current_shape.c.push(candidate)
			@current_shape.repaint
		else
			candidate.position.z = candidate.position.z + Constants::HEIGHT_S
			if !@current_shape.esta(candidate)
				@current_shape.c[position].erase
				@current_shape.c.delete_at(position)
				@current_shape.c.push(candidate)
				@current_shape.repaint
			else
				UI.messagebox("La semilla no se puede mover porque colisiona con otras figuras")
			end
		end
	end
	
	#pos:: the index of a specific shape
	#
	#Delete a specified shape from the history
	def delete_shape_history (pos, turned=false)
		cont = 0
		first = true
		while (cont < @execution_history.length)
			if @execution_history[cont][1].type.include? "triangle"
				position = @execution_history[cont][1].position(pos, turned)
			else
				position = @execution_history[cont][1].position(pos)
			end
			if position < @execution_history[cont][1].c.length && first
				if cont == 0
					@execution_history[cont][1].c.delete_at(position)
					cont += 1
				else
					@execution_history.delete_at(cont)
					first = false
					i = 0
					cont2 = 0
					while (i < @num_steps.length)&&(cont2 + @num_steps[i] < cont)
						cont2 += @num_steps[i]
						i += 1
					end
					num = @num_steps[i]
					if (num!= 0)
						@num_steps[i] = num - 1
					else
						@num_steps.delete_at(i)
						@num_steps[i - 1] = @num_steps[i - 1] - 1
					end
				end
			else
				@execution_history[cont][1].c.delete_at(position)
				cont += 1
			end
		end
		if @execution_history.length < 1
			@apply_r = false
		end
	end
	
	#Resets the execution, so the current shape becomes the axiom again
	def reset
		@execution_history = Array.new
		if @current_shape
			@current_shape.erase
		end
		@apply_r = false
		
		if @current_shape.type.include? "cube"
			cube = Cube.new(Constants::SIZE)
			@grammar.axiom = PracticaUtils.create_default_axiom(cube)
			@current_shape = PracticaUtils.create_current_shape_cube(cube)
		elsif @current_shape.type.include? "triangle"
			prism2 = TriangularPrism.new(Constants::SIZE)
			@grammar.axiom = PracticaUtils.create_default_axiom(prism2)
			@current_shape = PracticaUtils.create_current_shape_triangularPrism(prism2)
		elsif @current_shape.type.include? "hexagonal"
			prism2 = HexagonalPrism.new(Constants::SIZE_H)
			@grammar.axiom = PracticaUtils.create_default_axiom(prism2)
			@current_shape = PracticaUtils.create_current_shape_hexagonalPrism(prism2)
		end
		@axiom = @current_shape.c[0]
	end
	
	#Undoes the last taken step (that is, the current shape becomes the one that existed when the last rule had not been applied)
	def undo
		pair = @execution_history.pop
		if pair
			@current_shape.erase
			@current_shape = pair[1]
			@current_shape.repaint
			num = @num_steps.pop
			if num
				if (num!= 0)
					@num_steps.push(num-1)
				else
					num = @num_steps.pop
					@num_steps.push(num-1)
				end
			end
		end
		if @execution_history.length < 1
			@apply_r = false
		end
	end
	
	#Undoes the last taken step ()
	def undo_n
		cont = 0
		steps = @num_steps.pop
		if steps
			if steps == 0
				steps = @num_steps.pop
			end
			while cont < steps
				pair = @execution_history.pop
				cont+=1
			end
			if pair
				@current_shape.erase
				@current_shape = pair[1]
				@current_shape.repaint
			end
		end
		if @execution_history.length < 1
			@apply_r = false
		end
	end
end

#This class gathers an Execution object and the necessary information in order to save it in a file
class Project
	
	#The Execution object
	attr_accessor :execution
	#The title
	attr_reader :title
	#true if the view is been refreshed
	attr_accessor :modifying
	#true iff a rule shape is being deleted
	attr_accessor :erasing
	
	#Initializing
	def initialize(execution)
		@execution = execution
		
		@title = Constants::DEFAULT_PROJECT_TITLE
		
		@modifying = false
		
		@erasing = false

	end
	
	#new_title:: String with the new title for the project
	#closing:: true iff SketchUp is being closed
	#
	#Changes the title of the project to new_title
	def set_title(new_title, closing = false)
		@title = title
		if Shade.using_sketchup
			if !closing
				if @title_text
					Sketchup.active_model.entities.erase_entities @title_text
				end
			end
		end
	end
	
	#force:: true if we want to force refreshing, even when no changes have been registered
	#
	#Refreshes the view of the shapes in the project (that is, the rule shapes and the current shape)
	def refresh(force = false)

		@modifying = true
		if @execution
			@execution.current_shape.repaint
			if @execution.grammar
				@execution.grammar.rules.each {|rule|
					rule.paint(@execution)
				}
			end
		end
		@modifying = false

	end
end


#This class wrapps some global variables
class Practica

	#returns:: the current Project object
	def Practica.project
		return @@project
	end
	
	#project:: a Project object
	#
	#Sets the project
	def Practica.project=(project)
		@@project = project
	end
end

